# vim: set fileencoding=utf-8:
#
# GPIO Zero: a library for controlling the Raspberry Pi's GPIO pins
#
# Copyright (c) 2016-2021 Dave Jones <dave@waveform.org.uk>
# Copyright (c) 2020 Jack Wearden <jack@jackwearden.co.uk>
# Copyright (c) 2017-2020 Ben Nuttall <ben@bennuttall.com>
# Copyright (c) 2016-2019 Andrew Scheller <github@loowis.durge.org>
# Copyright (c) 2018 SteveAmor <steveamor@users.noreply.github.com>
# Copyright (c) 2018 Claire Pollard <claire.r.pollard@gmail.com>
# Copyright (c) 2016 Martin OHanlon <martin@ohanlonweb.com>
# Copyright (c) 2016 Ian Harcombe <ian.harcombe@gmail.com>
# Copyright (c) 2016 Andrew Scheller <lurch@durge.org>
#
# SPDX-License-Identifier: BSD-3-Clause

from __future__ import (
    unicode_literals,
    absolute_import,
    print_function,
    division,
    )
str = type('')


import sys
import pytest
from time import sleep
from threading import Event

import mock

from gpiozero import *
from gpiozero.fonts import *


def test_composite_output_on_off(mock_factory):
    pins = [mock_factory.pin(n) for n in (2, 3, 4)]
    with CompositeOutputDevice(OutputDevice(2), OutputDevice(3),
                               foo=OutputDevice(4)) as device:
        assert repr(device).startswith('<gpiozero.CompositeOutputDevice object')
        device.on()
        assert all(pin.state for pin in pins)
        device.off()
        assert not any(pin.state for pin in pins)

def test_composite_output_toggle(mock_factory):
    pin1 = mock_factory.pin(2)
    pin2 = mock_factory.pin(3)
    pin3 = mock_factory.pin(4)
    with CompositeOutputDevice(OutputDevice(2), OutputDevice(3),
                               foo=OutputDevice(4)) as device:
        device.toggle()
        assert all((pin1.state, pin2.state, pin3.state))
        device[0].off()
        device.toggle()
        assert pin1.state
        assert not pin2.state
        assert not pin3.state

def test_composite_output_value(mock_factory):
    pin1 = mock_factory.pin(2)
    pin2 = mock_factory.pin(3)
    pin3 = mock_factory.pin(4)
    with CompositeOutputDevice(OutputDevice(2), OutputDevice(3),
                               foo=OutputDevice(4)) as device:
        assert device.value == (0, 0, 0)
        device.toggle()
        assert device.value == (1, 1, 1)
        device.value = (1, 0, 1)
        assert device[0].is_active
        assert not device[1].is_active
        assert device[2].is_active

def test_button_board_bad_init(mock_factory):
    with pytest.raises(GPIOPinMissing):
        ButtonBoard()
    with pytest.raises(GPIOPinMissing):
        ButtonBoard(hold_repeat=True)

def test_button_board_init(mock_factory):
    pins = [mock_factory.pin(n) for n in (2, 3, 4)]
    with ButtonBoard(2, 3, 4) as board:
        assert repr(board).startswith('<gpiozero.ButtonBoard object')
        assert len(board) == 3
        assert isinstance(board[0], Button)
        assert isinstance(board[1], Button)
        assert isinstance(board[2], Button)
        assert board.pull_up
        assert board[0].pull_up
        assert board[1].pull_up
        assert board[2].pull_up
        assert [b.pin for b in board] == pins

def test_button_board_pressed_released(mock_factory):
    pin1 = mock_factory.pin(4)
    pin2 = mock_factory.pin(5)
    pin3 = mock_factory.pin(6)
    with ButtonBoard(4, 5, foo=6) as board:
        assert board.value == (False, False, False)
        assert board.wait_for_release(1)
        assert not board.is_active
        pin1.drive_low()
        pin3.drive_low()
        assert board.value == (True, False, True)
        assert board.wait_for_press(1)
        assert board.is_active
        pin3.drive_high()
        assert board.value == (True, False, False)
        assert board.is_active

def test_button_board_when_pressed(mock_factory):
    pin1 = mock_factory.pin(4)
    pin2 = mock_factory.pin(5)
    pin3 = mock_factory.pin(6)
    evt = Event()
    evt1 = Event()
    evt2 = Event()
    evt3 = Event()
    with ButtonBoard(4, 5, foo=6) as board:
        board.when_changed = evt.set
        board[0].when_pressed = evt1.set
        board[1].when_pressed = evt2.set
        board[2].when_pressed = evt3.set
        pin1.drive_low()
        assert evt.wait(1)
        assert evt1.wait(1)
        assert not evt2.wait(0)
        assert not evt3.wait(0)
        evt.clear()
        evt1.clear()
        pin1.drive_high()
        assert evt.wait(1)
        evt.clear()
        pin2.drive_low()
        assert evt.wait(1)
        assert evt2.wait(1)
        assert not evt1.wait(0)
        assert not evt3.wait(0)
        # Fake a pin event with no changes and make sure nothing gets set
        evt.clear()
        evt2.clear()
        pin2._call_when_changed()
        assert not evt.wait(0)
        assert not evt1.wait(0)
        assert not evt2.wait(0)
        assert not evt3.wait(0)

def test_led_collection_bad_init(mock_factory):
    with pytest.raises(GPIOPinMissing):
        LEDCollection()
    with pytest.raises(GPIOPinMissing):
        LEDCollection(pwm=True)

def test_led_collection_init(mock_factory):
    pins = [mock_factory.pin(n) for n in (2, 3, 4)]
    with LEDCollection(2, 3, 4) as board:
        assert repr(board).startswith('<gpiozero.LEDCollection object')
        assert len(board) == 3
        assert isinstance(board[0], LED)
        assert isinstance(board[1], LED)
        assert isinstance(board[2], LED)
        assert [b.pin for b in board] == pins

def test_led_board_bad_init(mock_factory):
    with pytest.raises(GPIOPinMissing):
        LEDBoard()
    with pytest.raises(GPIOPinMissing):
        LEDBoard(pwm=True)

def test_led_board_pwm_init(mock_factory, pwm):
    pins = [mock_factory.pin(n) for n in (2, 3, 4)]
    with LEDBoard(2, 3, 4, pwm=True) as board:
        assert repr(board).startswith('<gpiozero.LEDBoard object')
        assert len(board) == 3
        assert isinstance(board[0], PWMLED)
        assert isinstance(board[1], PWMLED)
        assert isinstance(board[2], PWMLED)
        assert [b.pin for b in board] == pins

def test_led_board_on_off(mock_factory):
    pin1 = mock_factory.pin(2)
    pin2 = mock_factory.pin(3)
    pin3 = mock_factory.pin(4)
    with LEDBoard(2, 3, foo=4) as board:
        assert repr(board).startswith('<gpiozero.LEDBoard object')
        assert isinstance(board[0], LED)
        assert isinstance(board[1], LED)
        assert isinstance(board[2], LED)
        assert board.active_high
        assert board[0].active_high
        assert board[1].active_high
        assert board[2].active_high
        board.on()
        assert all((pin1.state, pin2.state, pin3.state))
        board.off()
        assert not any((pin1.state, pin2.state, pin3.state))
        board[0].on()
        assert board.value == (1, 0, 0)
        assert pin1.state
        assert not pin2.state
        assert not pin3.state
        board.toggle()
        assert board.value == (0, 1, 1)
        assert not pin1.state
        assert pin2.state
        assert pin3.state
        board.toggle(0,1)
        assert board.value == (1, 0, 1)
        assert pin1.state
        assert not pin2.state
        assert pin3.state
        board.off(2)
        assert board.value == (1, 0, 0)
        assert pin1.state
        assert not pin2.state
        assert not pin3.state
        board.on(1)
        assert board.value == (1, 1, 0)
        assert pin1.state
        assert pin2.state
        assert not pin3.state
        board.off(0,1)
        assert board.value == (0, 0, 0)
        assert not pin1.state
        assert not pin2.state
        assert not pin3.state
        board.on(1,2)
        assert board.value == (0, 1, 1)
        assert not pin1.state
        assert pin2.state
        assert pin3.state
        board.toggle(0)
        assert board.value == (1, 1, 1)
        assert pin1.state
        assert pin2.state
        assert pin3.state

def test_led_board_active_low(mock_factory):
    pin1 = mock_factory.pin(2)
    pin2 = mock_factory.pin(3)
    pin3 = mock_factory.pin(4)
    with LEDBoard(2, 3, foo=4, active_high=False) as board:
        assert not board.active_high
        assert not board[0].active_high
        assert not board[1].active_high
        assert not board[2].active_high
        board.on()
        assert not any ((pin1.state, pin2.state, pin3.state))
        board.off()
        assert all((pin1.state, pin2.state, pin3.state))
        board[0].on()
        assert board.value == (1, 0, 0)
        assert not pin1.state
        assert pin2.state
        assert pin3.state
        board.toggle()
        assert board.value == (0, 1, 1)
        assert pin1.state
        assert not pin2.state
        assert not pin3.state

def test_led_board_value(mock_factory):
    pin1 = mock_factory.pin(2)
    pin2 = mock_factory.pin(3)
    pin3 = mock_factory.pin(4)
    with LEDBoard(2, 3, foo=4) as board:
        assert board.value == (0, 0, 0)
        board.value = (0, 1, 0)
        assert board.value == (0, 1, 0)
        board.value = (1, 0, 1)
        assert board.value == (1, 0, 1)

def test_led_board_pwm_value(mock_factory, pwm):
    pin1 = mock_factory.pin(2)
    pin2 = mock_factory.pin(3)
    pin3 = mock_factory.pin(4)
    with LEDBoard(2, 3, foo=4, pwm=True) as board:
        assert board.value == (0, 0, 0)
        board.value = (0, 1, 0)
        assert board.value == (0, 1, 0)
        board.value = (0.5, 0, 0.75)
        assert board.value == (0.5, 0, 0.75)

def test_led_board_pwm_bad_value(mock_factory, pwm):
    pin1 = mock_factory.pin(2)
    pin2 = mock_factory.pin(3)
    pin3 = mock_factory.pin(4)
    with LEDBoard(2, 3, foo=4, pwm=True) as board:
        with pytest.raises(ValueError):
            board.value = (-1, 0, 0)
        with pytest.raises(ValueError):
            board.value = (0, 2, 0)

def test_led_board_initial_value(mock_factory):
    pin1 = mock_factory.pin(2)
    pin2 = mock_factory.pin(3)
    pin3 = mock_factory.pin(4)
    with LEDBoard(2, 3, foo=4, initial_value=0) as board:
        assert board.value == (0, 0, 0)
    with LEDBoard(2, 3, foo=4, initial_value=1) as board:
        assert board.value == (1, 1, 1)

def test_led_board_pwm_initial_value(mock_factory, pwm):
    pin1 = mock_factory.pin(2)
    pin2 = mock_factory.pin(3)
    pin3 = mock_factory.pin(4)
    with LEDBoard(2, 3, foo=4, pwm=True, initial_value=0) as board:
        assert board.value == (0, 0, 0)
    with LEDBoard(2, 3, foo=4, pwm=True, initial_value=1) as board:
        assert board.value == (1, 1, 1)
    with LEDBoard(2, 3, foo=4, pwm=True, initial_value=0.5) as board:
        assert board.value == (0.5, 0.5, 0.5)

def test_led_board_pwm_bad_initial_value(mock_factory, pwm):
    pin1 = mock_factory.pin(2)
    pin2 = mock_factory.pin(3)
    pin3 = mock_factory.pin(4)
    with pytest.raises(ValueError):
        LEDBoard(2, 3, foo=4, pwm=True, initial_value=-1)
    with pytest.raises(ValueError):
        LEDBoard(2, 3, foo=4, pwm=True, initial_value=2)

def test_led_board_nested(mock_factory):
    pin1 = mock_factory.pin(2)
    pin2 = mock_factory.pin(3)
    pin3 = mock_factory.pin(4)
    with LEDBoard(2, LEDBoard(3, 4)) as board:
        assert list(led.pin for led in board.leds) == [pin1, pin2, pin3]
        assert board.value == (0, (0, 0))
        board.value = (1, (0, 1))
        assert pin1.state
        assert not pin2.state
        assert pin3.state

def test_led_board_bad_blink(mock_factory):
    pin1 = mock_factory.pin(2)
    pin2 = mock_factory.pin(3)
    pin3 = mock_factory.pin(4)
    with LEDBoard(2, LEDBoard(3, 4)) as board:
        with pytest.raises(ValueError):
            board.blink(fade_in_time=1, fade_out_time=1)
        with pytest.raises(ValueError):
            board.blink(fade_out_time=1)
        with pytest.raises(ValueError):
            board.pulse()

@pytest.mark.skipif(hasattr(sys, 'pypy_version_info'),
                    reason='timing is too random on pypy')
def test_led_board_blink_background(mock_factory):
    pin1 = mock_factory.pin(4)
    pin2 = mock_factory.pin(5)
    pin3 = mock_factory.pin(6)
    with LEDBoard(4, LEDBoard(5, 6)) as board:
        # Instantiation takes a long enough time that it throws off our timing
        # here!
        pin1.clear_states()
        pin2.clear_states()
        pin3.clear_states()
        board.blink(0.1, 0.1, n=2)
        board._blink_thread.join() # naughty, but ensures no arbitrary waits in the test
        test = [
            (0.0, False),
            (0.0, True),
            (0.1, False),
            (0.1, True),
            (0.1, False)
            ]
        pin1.assert_states_and_times(test)
        pin2.assert_states_and_times(test)
        pin3.assert_states_and_times(test)

@pytest.mark.skipif(hasattr(sys, 'pypy_version_info'),
                    reason='timing is too random on pypy')
def test_led_board_blink_foreground(mock_factory):
    pin1 = mock_factory.pin(4)
    pin2 = mock_factory.pin(5)
    pin3 = mock_factory.pin(6)
    with LEDBoard(4, LEDBoard(5, 6)) as board:
        pin1.clear_states()
        pin2.clear_states()
        pin3.clear_states()
        board.blink(0.1, 0.1, n=2, background=False)
        test = [
            (0.0, False),
            (0.0, True),
            (0.1, False),
            (0.1, True),
            (0.1, False)
            ]
        pin1.assert_states_and_times(test)
        pin2.assert_states_and_times(test)
        pin3.assert_states_and_times(test)

@pytest.mark.skipif(hasattr(sys, 'pypy_version_info'),
                    reason='timing is too random on pypy')
def test_led_board_blink_control(mock_factory):
    pin1 = mock_factory.pin(4)
    pin2 = mock_factory.pin(5)
    pin3 = mock_factory.pin(6)
    with LEDBoard(4, LEDBoard(5, 6)) as board:
        pin1.clear_states()
        pin2.clear_states()
        pin3.clear_states()
        board.blink(0.1, 0.1, n=2)
        # make sure the blink thread's started
        while not board._blink_leds:
            sleep(0.00001) # pragma: no cover
        board[1][0].off() # immediately take over the second LED
        board._blink_thread.join() # naughty, but ensures no arbitrary waits in the test
        test = [
            (0.0, False),
            (0.0, True),
            (0.1, False),
            (0.1, True),
            (0.1, False)
            ]
        pin1.assert_states_and_times(test)
        pin3.assert_states_and_times(test)
        print(pin2.states)
        pin2.assert_states_and_times([(0.0, False), (0.0, True), (0.0, False)])

@pytest.mark.skipif(hasattr(sys, 'pypy_version_info'),
                    reason='timing is too random on pypy')
def test_led_board_blink_take_over(mock_factory):
    pin1 = mock_factory.pin(4)
    pin2 = mock_factory.pin(5)
    pin3 = mock_factory.pin(6)
    with LEDBoard(4, LEDBoard(5, 6)) as board:
        pin1.clear_states()
        pin2.clear_states()
        pin3.clear_states()
        board[1].blink(0.1, 0.1, n=2)
        board.blink(0.1, 0.1, n=2) # immediately take over blinking
        board[1]._blink_thread.join()
        board._blink_thread.join()
        test = [
            (0.0, False),
            (0.0, True),
            (0.1, False),
            (0.1, True),
            (0.1, False)
            ]
        pin1.assert_states_and_times(test)
        pin2.assert_states_and_times(test)
        pin3.assert_states_and_times(test)

@pytest.mark.skipif(hasattr(sys, 'pypy_version_info'),
                    reason='timing is too random on pypy')
def test_led_board_blink_control_all(mock_factory):
    pin1 = mock_factory.pin(4)
    pin2 = mock_factory.pin(5)
    pin3 = mock_factory.pin(6)
    with LEDBoard(4, LEDBoard(5, 6)) as board:
        pin1.clear_states()
        pin2.clear_states()
        pin3.clear_states()
        board.blink(0.1, 0.1, n=2)
        # make sure the blink thread's started
        while not board._blink_leds:
            sleep(0.00001) # pragma: no cover
        board[0].off() # immediately take over all LEDs
        board[1][0].off()
        board[1][1].off()
        board._blink_thread.join() # blink should terminate here anyway
        test = [
            (0.0, False),
            (0.0, True),
            (0.0, False),
            ]
        pin1.assert_states_and_times(test)
        pin2.assert_states_and_times(test)
        pin3.assert_states_and_times(test)

def test_led_board_blink_interrupt_on(mock_factory):
    pin1 = mock_factory.pin(4)
    pin2 = mock_factory.pin(5)
    pin3 = mock_factory.pin(6)
    with LEDBoard(4, LEDBoard(5, 6)) as board:
        board.blink(1, 0.1)
        sleep(0.2)
        board.off() # should interrupt while on
        pin1.assert_states([False, True, False])
        pin2.assert_states([False, True, False])
        pin3.assert_states([False, True, False])

def test_led_board_blink_interrupt_off(mock_factory):
    pin1 = mock_factory.pin(4)
    pin2 = mock_factory.pin(5)
    pin3 = mock_factory.pin(6)
    with LEDBoard(4, LEDBoard(5, 6)) as board:
        pin1.clear_states()
        pin2.clear_states()
        pin3.clear_states()
        board.blink(0.1, 1)
        sleep(0.2)
        board.off() # should interrupt while off
        pin1.assert_states([False, True, False])
        pin2.assert_states([False, True, False])
        pin3.assert_states([False, True, False])

@pytest.mark.skipif(hasattr(sys, 'pypy_version_info'),
                    reason='timing is too random on pypy')
def test_led_board_fade_background(mock_factory, pwm):
    pin1 = mock_factory.pin(4)
    pin2 = mock_factory.pin(5)
    pin3 = mock_factory.pin(6)
    with LEDBoard(4, LEDBoard(5, 6, pwm=True), pwm=True) as board:
        pin1.clear_states()
        pin2.clear_states()
        pin3.clear_states()
        board.blink(0, 0, 0.2, 0.2, n=2)
        board._blink_thread.join()
        test = [
            (0.0, 0),
            (0.04, 0.2),
            (0.04, 0.4),
            (0.04, 0.6),
            (0.04, 0.8),
            (0.04, 1),
            (0.04, 0.8),
            (0.04, 0.6),
            (0.04, 0.4),
            (0.04, 0.2),
            (0.04, 0),
            (0.04, 0.2),
            (0.04, 0.4),
            (0.04, 0.6),
            (0.04, 0.8),
            (0.04, 1),
            (0.04, 0.8),
            (0.04, 0.6),
            (0.04, 0.4),
            (0.04, 0.2),
            (0.04, 0),
            ]
        pin1.assert_states_and_times(test)
        pin2.assert_states_and_times(test)
        pin3.assert_states_and_times(test)

def test_led_bar_graph_value(mock_factory):
    pin1 = mock_factory.pin(2)
    pin2 = mock_factory.pin(3)
    pin3 = mock_factory.pin(4)
    with LEDBarGraph(2, 3, 4) as graph:
        assert repr(graph).startswith('<gpiozero.LEDBarGraph object')
        assert isinstance(graph[0], LED)
        assert isinstance(graph[1], LED)
        assert isinstance(graph[2], LED)
        assert graph.active_high
        assert graph[0].active_high
        assert graph[1].active_high
        assert graph[2].active_high
        graph.value = 0
        assert graph.value == 0
        assert graph.lit_count == 0
        assert not any((pin1.state, pin2.state, pin3.state))
        graph.value = 1
        assert graph.value == 1
        assert graph.lit_count == 3
        assert all((pin1.state, pin2.state, pin3.state))
        graph.value = 1/3
        assert graph.value == 1/3
        assert graph.lit_count == 1
        assert pin1.state and not (pin2.state or pin3.state)
        graph.value = -1/3
        assert graph.value == -1/3
        assert graph.lit_count == -1
        assert pin3.state and not (pin1.state or pin2.state)
        pin1.state = True
        pin2.state = True
        assert graph.value == 1
        assert graph.lit_count == 3
        pin3.state = False
        assert graph.value == 2/3
        assert graph.lit_count == 2
        pin3.state = True
        pin1.state = False
        assert graph.value == -2/3
        graph.lit_count = 2
        assert graph.value == 2/3
        graph.lit_count = -1
        assert graph.value == -1/3
        graph.lit_count = -3
        assert graph.value == 1

def test_led_bar_graph_active_low(mock_factory):
    pin1 = mock_factory.pin(2)
    pin2 = mock_factory.pin(3)
    pin3 = mock_factory.pin(4)
    with LEDBarGraph(2, 3, 4, active_high=False) as graph:
        assert not graph.active_high
        assert not graph[0].active_high
        assert not graph[1].active_high
        assert not graph[2].active_high
        graph.value = 0
        assert graph.value == 0
        assert all((pin1.state, pin2.state, pin3.state))
        graph.value = 1
        assert graph.value == 1
        assert not any((pin1.state, pin2.state, pin3.state))
        graph.value = 1/3
        assert graph.value == 1/3
        assert not pin1.state and pin2.state and pin3.state
        graph.value = -1/3
        assert graph.value == -1/3
        assert not pin3.state and pin1.state and pin2.state

def test_led_bar_graph_pwm_value(mock_factory, pwm):
    pin1 = mock_factory.pin(2)
    pin2 = mock_factory.pin(3)
    pin3 = mock_factory.pin(4)
    pins = (pin1, pin2, pin3)
    with LEDBarGraph(2, 3, 4, pwm=True) as graph:
        assert repr(graph).startswith('<gpiozero.LEDBarGraph object')
        assert isinstance(graph[0], PWMLED)
        assert isinstance(graph[1], PWMLED)
        assert isinstance(graph[2], PWMLED)
        graph.value = 0
        assert graph.value == 0
        assert graph.lit_count == 0
        assert not any(pin.state for pin in pins)
        graph.value = 1
        assert graph.value == 1
        assert graph.lit_count == 3
        assert all(pin.state for pin in pins)
        graph.value = 1/3
        assert graph.value == 1/3
        assert graph.lit_count == 1
        assert pin1.state and not (pin2.state or pin3.state)
        graph.value = -1/3
        assert graph.value == -1/3
        assert graph.lit_count == -1
        assert pin3.state and not (pin1.state or pin2.state)
        graph.value = 1/2
        assert graph.value == 1/2
        assert graph.lit_count == 1.5
        assert (pin1.state, pin2.state, pin3.state) == (1, 0.5, 0)
        pin1.state = 0
        pin3.state = 1
        assert graph.value == -1/2
        assert graph.lit_count == -1.5
        graph.lit_count = 1.5
        assert graph.value == 0.5

def test_led_bar_graph_bad_value(mock_factory):
    pin1 = mock_factory.pin(2)
    pin2 = mock_factory.pin(3)
    pin3 = mock_factory.pin(4)
    with LEDBarGraph(2, 3, 4) as graph:
        with pytest.raises(ValueError):
            graph.value = -2
        with pytest.raises(ValueError):
            graph.value = 2
        with pytest.raises(ValueError):
            graph.lit_count = -4
        with pytest.raises(ValueError):
            graph.lit_count = 4

def test_led_bar_graph_bad_init(mock_factory):
    pin1 = mock_factory.pin(2)
    pin2 = mock_factory.pin(3)
    pin3 = mock_factory.pin(4)
    with pytest.raises(GPIOPinMissing):
        LEDBarGraph()
    with pytest.raises(GPIOPinMissing):
        LEDBarGraph(pwm=True)
    with pytest.raises(TypeError):
        LEDBarGraph(2, 3, foo=4)
    with pytest.raises(ValueError):
        LEDBarGraph(2, 3, 4, initial_value=-2)
    with pytest.raises(ValueError):
        LEDBarGraph(2, 3, 4, initial_value=2)
    with pytest.raises(ValueError):
        LEDBarGraph(2, LEDBoard(3, 4))

def test_led_bar_graph_initial_value(mock_factory):
    pin1 = mock_factory.pin(2)
    pin2 = mock_factory.pin(3)
    pin3 = mock_factory.pin(4)
    with LEDBarGraph(2, 3, 4, initial_value=1/3) as graph:
        assert graph.value == 1/3
        assert pin1.state and not (pin2.state or pin3.state)
    with LEDBarGraph(2, 3, 4, initial_value=-1/3) as graph:
        assert graph.value == -1/3
        assert pin3.state and not (pin1.state or pin2.state)

def test_led_bar_graph_pwm_initial_value(mock_factory, pwm):
    pin1 = mock_factory.pin(2)
    pin2 = mock_factory.pin(3)
    pin3 = mock_factory.pin(4)
    with LEDBarGraph(2, 3, 4, pwm=True, initial_value=0.5) as graph:
        assert graph.value == 0.5
        assert (pin1.state, pin2.state, pin3.state) == (1, 0.5, 0)
    with LEDBarGraph(2, 3, 4, pwm=True, initial_value=-0.5) as graph:
        assert graph.value == -0.5
        assert (pin1.state, pin2.state, pin3.state) == (0, 0.5, 1)

def test_led_borg(mock_factory, pwm):
    pins = [mock_factory.pin(n) for n in (17, 27, 22)]
    with LedBorg() as board:
        assert repr(board).startswith('<gpiozero.LedBorg object')
        assert [device.pin for device in board._leds] == pins

def test_pi_liter(mock_factory):
    pins = [mock_factory.pin(n) for n in (4, 17, 27, 18, 22, 23, 24, 25)]
    with PiLiter() as board:
        assert repr(board).startswith('<gpiozero.PiLiter object')
        assert [device.pin for device in board] == pins

def test_pi_liter_graph(mock_factory):
    pins = [mock_factory.pin(n) for n in (4, 17, 27, 18, 22, 23, 24, 25)]
    with PiLiterBarGraph() as board:
        assert repr(board).startswith('<gpiozero.PiLiterBarGraph object')
        board.value = 0.5
        assert [pin.state for pin in pins] == [1, 1, 1, 1, 0, 0, 0, 0]
        pins[4].state = 1
        assert board.value == 5/8

def test_traffic_lights(mock_factory):
    red_pin = mock_factory.pin(2)
    amber_pin = mock_factory.pin(3)
    green_pin = mock_factory.pin(4)
    with TrafficLights(2, 3, 4) as board:
        assert repr(board).startswith('<gpiozero.TrafficLights object')
        board.red.on()
        assert board.red.value
        assert not board.amber.value
        assert not board.yellow.value
        assert not board.green.value
        assert red_pin.state
        assert not amber_pin.state
        assert not green_pin.state
        board.amber.on()
        assert amber_pin.state
        board.yellow.off()
        assert not amber_pin.state
    with TrafficLights(red=2, yellow=3, green=4) as board:
        board.yellow.on()
        assert not board.red.value
        assert board.amber.value
        assert board.yellow.value
        assert not board.green.value
        assert not red_pin.state
        assert amber_pin.state
        assert not green_pin.state
        board.amber.off()
        assert not amber_pin.state

def test_traffic_lights_bad_init(mock_factory):
    with pytest.raises(GPIOPinMissing):
        TrafficLights()
    with pytest.raises(GPIOPinMissing):
        TrafficLights(2)
    with pytest.raises(GPIOPinMissing):
        TrafficLights(2, 3)
    red_pin = mock_factory.pin(2)
    amber_pin = mock_factory.pin(3)
    green_pin = mock_factory.pin(4)
    yellow_pin = mock_factory.pin(5)
    with pytest.raises(ValueError):
        TrafficLights(red=2, amber=3, yellow=5, green=4)

def test_pi_traffic(mock_factory):
    pins = [mock_factory.pin(n) for n in (9, 10, 11)]
    with PiTraffic() as board:
        assert repr(board).startswith('<gpiozero.PiTraffic object')
        assert [device.pin for device in board] == pins

def test_pi_stop_bad_init(mock_factory):
    with pytest.raises(ValueError):
        PiStop()
    with pytest.raises(ValueError):
        PiStop('E')

def test_pi_stop(mock_factory):
    pins_a = [mock_factory.pin(n) for n in (7, 8, 25)]
    with PiStop('A') as board:
        assert repr(board).startswith('<gpiozero.PiStop object')
        assert [device.pin for device in board] == pins_a
    pins_aplus = [mock_factory.pin(n) for n in (21, 20, 16)]
    with PiStop('A+') as board:
        assert [device.pin for device in board] == pins_aplus
    pins_b = [mock_factory.pin(n) for n in (10, 9, 11)]
    with PiStop('B') as board:
        assert [device.pin for device in board] == pins_b
    pins_bplus = [mock_factory.pin(n) for n in (13, 19, 26)]
    with PiStop('B+') as board:
        assert [device.pin for device in board] == pins_bplus
    pins_c = [mock_factory.pin(n) for n in (18, 15, 14)]
    with PiStop('C') as board:
        assert [device.pin for device in board] == pins_c
    pins_d = [mock_factory.pin(n) for n in (2, 3, 4)]
    with PiStop('D') as board:
        assert [device.pin for device in board] == pins_d

def test_snow_pi(mock_factory):
    pins = [mock_factory.pin(n) for n in (23, 24, 25, 17, 18, 22, 7, 8, 9)]
    with SnowPi() as board:
        assert repr(board).startswith('<gpiozero.SnowPi object')
        assert [device.pin for device in board.leds] == pins

def test_snow_pi_initial_value(mock_factory):
    with SnowPi() as board:
        assert all(device.pin.state == False for device in board.leds)
    with SnowPi(initial_value=False) as board:
        assert all(device.pin.state == False for device in board.leds)
    with SnowPi(initial_value=True) as board:
        assert all(device.pin.state == True for device in board.leds)
    with SnowPi(initial_value=0.5) as board:
        assert all(device.pin.state == True for device in board.leds)

def test_snow_pi_initial_value_pwm(mock_factory, pwm):
    pins = [mock_factory.pin(n) for n in (23, 24, 25, 17, 18, 22, 7, 8, 9)]
    with SnowPi(pwm=True, initial_value=0.5) as board:
        assert [device.pin for device in board.leds] == pins
        assert all(device.pin.state == 0.5 for device in board.leds)

def test_pihut_xmas_tree(mock_factory):
    led_pins = (2, 4, 15, 13, 21, 25, 8, 5, 10, 16, 17, 27, 26,
                24, 9, 12, 6, 20, 19, 14, 18, 11, 7, 23, 22)
    pins = [mock_factory.pin(n) for n in led_pins]
    with PiHutXmasTree() as tree:
        assert repr(tree).startswith('<gpiozero.PiHutXmasTree object')
        assert [led.pin for led in tree.leds] == pins
        assert len(tree) == 25
        assert isinstance(tree.star, LED)
        assert isinstance(tree.led1, LED)
        assert isinstance(tree.led24, LED)

def test_pihut_xmas_tree_init(mock_factory):
    with PiHutXmasTree(pwm=False) as tree:
        assert isinstance(tree.star, LED)
        assert isinstance(tree.led1, LED)
        assert isinstance(tree.led24, LED)
    with PiHutXmasTree(initial_value=True) as tree:
        assert all(led.value for led in tree)

def test_pihut_xmas_tree_pwm(mock_factory, pwm):
    with PiHutXmasTree(pwm=True) as tree:
        assert isinstance(tree.star, PWMLED)
        assert isinstance(tree.led1, PWMLED)
        assert isinstance(tree.led24, PWMLED)
    with PiHutXmasTree(pwm=True, initial_value=0.5) as tree:
        assert all(led.value == 0.5 for led in tree)

def test_pihut_xmas_tree_leds(mock_factory):
    with PiHutXmasTree() as tree:
        tree.star.on()
        assert sum(tree.value) == 1
        tree.led1.on()
        assert sum(tree.value) == 2
        tree.on()
        assert sum(tree.value) == 25
        tree.off()
        assert sum(tree.value) == 0
        tree.toggle()
        assert sum(tree.value) == 25

def test_traffic_lights_buzzer(mock_factory):
    red_pin = mock_factory.pin(2)
    amber_pin = mock_factory.pin(3)
    green_pin = mock_factory.pin(4)
    buzzer_pin = mock_factory.pin(5)
    button_pin = mock_factory.pin(6)
    with TrafficLightsBuzzer(
            TrafficLights(2, 3, 4),
            Buzzer(5),
            Button(6)) as board:
        assert repr(board).startswith('<gpiozero.TrafficLightsBuzzer object')
        board.lights.red.on()
        board.buzzer.on()
        assert red_pin.state
        assert not amber_pin.state
        assert not green_pin.state
        assert buzzer_pin.state
        button_pin.drive_low()
        assert board.button.is_active
        board.toggle()
        assert not red_pin.state
        assert amber_pin.state
        assert green_pin.state
        assert not buzzer_pin.state
        assert board.value == ((0, 1, 1), 0, 1)
        board.value = ((0, 0, 0), 1, 0)
        assert not red_pin.state
        assert not amber_pin.state
        assert not green_pin.state
        assert buzzer_pin.state

def test_fish_dish(mock_factory):
    pins = [mock_factory.pin(n) for n in (9, 22, 4, 8, 7)]
    with FishDish() as board:
        assert repr(board).startswith('<gpiozero.FishDish object')
        assert ([led.pin for led in board.lights] +
                [board.buzzer.pin, board.button.pin]) == pins

def test_traffic_hat(mock_factory):
    pins = [mock_factory.pin(n) for n in (24, 23, 22, 5, 25)]
    with TrafficHat() as board:
        assert repr(board).startswith('<gpiozero.TrafficHat object')
        assert ([led.pin for led in board.lights] +
                [board.buzzer.pin, board.button.pin]) == pins

def test_traffic_phat(mock_factory):
    pins = [mock_factory.pin(n) for n in (25, 24, 23)]
    with TrafficpHat() as board:
        assert repr(board).startswith('<gpiozero.TrafficpHat object')
        assert ([led.pin for led in board]) == pins

def test_robot_bad_init(mock_factory):
    with pytest.raises(GPIOPinMissing):
        Robot()
    with pytest.raises(GPIOPinMissing):
        Robot(2)
    with pytest.raises(GPIOPinMissing):
        Robot(2, 3)
    with pytest.raises(GPIOPinMissing):
        Robot(2, 3, 4)
    with pytest.raises(GPIOPinMissing):
        Robot(2, 3, 4, 5)
    with pytest.raises(GPIOPinMissing):
        Robot(2, 3, 4, 5, 6, 7)
    with pytest.raises(GPIOPinMissing):
        Robot((2, 3))
    with pytest.raises(GPIOPinMissing):
        Robot((2, 3), 4)

def test_robot(mock_factory, pwm):
    pins = [mock_factory.pin(n) for n in (2, 3, 4, 5)]
    def check_pins_and_value(robot, expected_value):
        # Ensure both forward and back pins aren't both driven simultaneously
        assert pins[0].state == 0 or pins[1].state == 0
        assert pins[2].state == 0 or pins[3].state == 0
        assert robot.value == (
            pins[0].state - pins[1].state, pins[2].state - pins[3].state
        ) == expected_value
    with Robot((2, 3), (4, 5)) as robot:
        assert repr(robot).startswith('<gpiozero.Robot object')
        assert (
            [device.pin for device in robot.left_motor] +
            [device.pin for device in robot.right_motor]) == pins
        check_pins_and_value(robot, (0, 0))
        robot.forward()
        check_pins_and_value(robot, (1, 1))
        robot.backward()
        check_pins_and_value(robot, (-1, -1))
        robot.forward(0)
        check_pins_and_value(robot, (0, 0))
        robot.forward(0.5)
        check_pins_and_value(robot, (0.5, 0.5))
        robot.forward(1)
        check_pins_and_value(robot, (1, 1))
        robot.forward(curve_right=0)
        check_pins_and_value(robot, (1, 1))
        robot.forward(curve_left=0)
        check_pins_and_value(robot, (1, 1))
        robot.forward(curve_left=0, curve_right=0)
        check_pins_and_value(robot, (1, 1))
        robot.forward(curve_right=1)
        check_pins_and_value(robot, (1, 0))
        robot.forward(curve_left=1)
        check_pins_and_value(robot, (0, 1))
        robot.forward(0.5, curve_right=1)
        check_pins_and_value(robot, (0.5, 0))
        robot.forward(0.5, curve_left=1)
        check_pins_and_value(robot, (0, 0.5))
        robot.forward(curve_right=0.5)
        check_pins_and_value(robot, (1, 0.5))
        robot.forward(curve_left=0.5)
        check_pins_and_value(robot, (0.5, 1))
        robot.forward(0.5, curve_right=0.5)
        check_pins_and_value(robot, (0.5, 0.25))
        robot.forward(0.5, curve_left=0.5)
        check_pins_and_value(robot, (0.25, 0.5))
        with pytest.raises(ValueError):
            robot.forward(-1)
        with pytest.raises(ValueError):
            robot.forward(2)
        with pytest.raises(ValueError):
            robot.forward(curve_left=-1)
        with pytest.raises(ValueError):
            robot.forward(curve_left=2)
        with pytest.raises(ValueError):
            robot.forward(curve_right=-1)
        with pytest.raises(ValueError):
            robot.forward(curve_right=2)
        with pytest.raises(ValueError):
            robot.forward(curve_left=1, curve_right=1)
        robot.backward()
        check_pins_and_value(robot, (-1, -1))
        robot.reverse()
        check_pins_and_value(robot, (1, 1))
        robot.backward(0)
        check_pins_and_value(robot, (0, 0))
        robot.backward(0.5)
        check_pins_and_value(robot, (-0.5, -0.5))
        robot.backward(1)
        check_pins_and_value(robot, (-1, -1))
        robot.backward(curve_right=0)
        check_pins_and_value(robot, (-1, -1))
        robot.backward(curve_left=0)
        check_pins_and_value(robot, (-1, -1))
        robot.backward(curve_left=0, curve_right=0)
        check_pins_and_value(robot, (-1, -1))
        robot.backward(curve_right=1)
        check_pins_and_value(robot, (-1, 0))
        robot.backward(curve_left=1)
        check_pins_and_value(robot, (0, -1))
        robot.backward(0.5, curve_right=1)
        check_pins_and_value(robot, (-0.5, 0))
        robot.backward(0.5, curve_left=1)
        check_pins_and_value(robot, (0, -0.5))
        robot.backward(curve_right=0.5)
        check_pins_and_value(robot, (-1, -0.5))
        robot.backward(curve_left=0.5)
        check_pins_and_value(robot, (-0.5, -1))
        robot.backward(0.5, curve_right=0.5)
        check_pins_and_value(robot, (-0.5, -0.25))
        robot.backward(0.5, curve_left=0.5)
        check_pins_and_value(robot, (-0.25, -0.5))
        with pytest.raises(ValueError):
            robot.backward(-1)
        with pytest.raises(ValueError):
            robot.backward(2)
        with pytest.raises(ValueError):
            robot.backward(curve_left=-1)
        with pytest.raises(ValueError):
            robot.backward(curve_left=2)
        with pytest.raises(ValueError):
            robot.backward(curve_right=-1)
        with pytest.raises(ValueError):
            robot.backward(curve_right=2)
        with pytest.raises(ValueError):
            robot.backward(curve_left=1, curve_right=1)
        with pytest.raises(TypeError):
            robot.forward(curveleft=1)
        with pytest.raises(TypeError):
            robot.backward(curveleft=1)
        robot.left()
        check_pins_and_value(robot, (-1, 1))
        robot.left(0)
        check_pins_and_value(robot, (0, 0))
        robot.left(0.5)
        check_pins_and_value(robot, (-0.5, 0.5))
        robot.left(1)
        check_pins_and_value(robot, (-1, 1))
        with pytest.raises(ValueError):
            robot.left(-1)
        with pytest.raises(ValueError):
            robot.left(2)
        robot.right()
        check_pins_and_value(robot, (1, -1))
        robot.right(0)
        check_pins_and_value(robot, (0, 0))
        robot.right(0.5)
        check_pins_and_value(robot, (0.5, -0.5))
        robot.right(1)
        check_pins_and_value(robot, (1, -1))
        with pytest.raises(ValueError):
            robot.right(-1)
        with pytest.raises(ValueError):
            robot.right(2)
        robot.reverse()
        check_pins_and_value(robot, (-1, 1))
        robot.stop()
        check_pins_and_value(robot, (0, 0))
        robot.stop()
        check_pins_and_value(robot, (0, 0))
        robot.value = (-1, -1)
        check_pins_and_value(robot, (-1, -1))
        robot.value = (0.5, 1)
        check_pins_and_value(robot, (0.5, 1))
        robot.value = (0, -0.5)
        check_pins_and_value(robot, (0, -0.5))

def test_robots(mock_factory, pwm):
    with RyanteckRobot() as robot:
        left_motor, right_motor = robot.all
        assert isinstance(left_motor, Motor)
        assert isinstance(left_motor.forward_device, PWMOutputDevice)
        assert isinstance(left_motor.backward_device, PWMOutputDevice)
        assert isinstance(right_motor, Motor)
        assert isinstance(right_motor.forward_device, PWMOutputDevice)
        assert isinstance(right_motor.backward_device, PWMOutputDevice)

    with CamJamKitRobot() as robot:
        left_motor, right_motor = robot.all
        assert isinstance(left_motor, Motor)
        assert isinstance(left_motor.forward_device, PWMOutputDevice)
        assert isinstance(left_motor.backward_device, PWMOutputDevice)
        assert isinstance(right_motor, Motor)
        assert isinstance(right_motor.forward_device, PWMOutputDevice)
        assert isinstance(right_motor.backward_device, PWMOutputDevice)

    with PololuDRV8835Robot() as robot:
        left_motor, right_motor = robot.all
        assert isinstance(left_motor, PhaseEnableMotor)
        assert isinstance(left_motor.phase_device, DigitalOutputDevice)
        assert isinstance(left_motor.enable_device, PWMOutputDevice)
        assert isinstance(right_motor, PhaseEnableMotor)
        assert isinstance(right_motor.phase_device, DigitalOutputDevice)
        assert isinstance(right_motor.enable_device, PWMOutputDevice)

def test_robot_nopwm(mock_factory):
    pins = [mock_factory.pin(n) for n in (2, 3, 4, 5)]
    with Robot((2, 3), (4, 5), pwm=False) as robot:
        left_motor, right_motor = robot.all
        assert isinstance(left_motor, Motor)
        assert left_motor.forward_device.pin is pins[0]
        assert isinstance(left_motor.forward_device, DigitalOutputDevice)
        assert left_motor.backward_device.pin is pins[1]
        assert isinstance(left_motor.forward_device, DigitalOutputDevice)
        assert isinstance(right_motor, Motor)
        assert right_motor.forward_device.pin is pins[2]
        assert isinstance(right_motor.forward_device, DigitalOutputDevice)
        assert right_motor.backward_device.pin is pins[3]
        assert isinstance(right_motor.backward_device, DigitalOutputDevice)

def test_robots_nopwm(mock_factory):
    pins = [mock_factory.pin(n) for n in (17, 18, 22, 23)]
    with RyanteckRobot(pwm=False) as robot:
        assert repr(robot).startswith('<gpiozero.RyanteckRobot object')
        assert [device.pin for motor in robot for device in motor] == pins
        left_motor, right_motor = robot.all
        assert isinstance(left_motor, Motor)
        assert isinstance(left_motor.forward_device, DigitalOutputDevice)
        assert isinstance(left_motor.backward_device, DigitalOutputDevice)
        assert isinstance(right_motor, Motor)
        assert isinstance(right_motor.forward_device, DigitalOutputDevice)
        assert isinstance(right_motor.backward_device, DigitalOutputDevice)

    pins = [mock_factory.pin(n) for n in (9, 10, 7, 8)]
    with CamJamKitRobot(pwm=False) as robot:
        assert repr(robot).startswith('<gpiozero.CamJamKitRobot object')
        assert [device.pin for motor in robot for device in motor] == pins
        left_motor, right_motor = robot.all
        assert isinstance(left_motor, Motor)
        assert isinstance(left_motor.forward_device, DigitalOutputDevice)
        assert isinstance(left_motor.backward_device, DigitalOutputDevice)
        assert isinstance(right_motor, Motor)
        assert isinstance(right_motor.forward_device, DigitalOutputDevice)
        assert isinstance(right_motor.backward_device, DigitalOutputDevice)

    pins = [mock_factory.pin(n) for n in (5, 12, 6, 13)]
    with PololuDRV8835Robot(pwm=False) as robot:
        assert repr(robot).startswith('<gpiozero.PololuDRV8835Robot object')
        assert [device.pin for motor in robot for device in motor] == pins
        left_motor, right_motor = robot.all
        assert isinstance(left_motor, PhaseEnableMotor)
        assert isinstance(left_motor.phase_device, DigitalOutputDevice)
        assert isinstance(left_motor.enable_device, DigitalOutputDevice)
        assert isinstance(right_motor, PhaseEnableMotor)
        assert isinstance(right_motor.phase_device, DigitalOutputDevice)
        assert isinstance(right_motor.enable_device, DigitalOutputDevice)

def test_enable_pin_motor_robot(mock_factory, pwm):
    pins = [mock_factory.pin(n) for n in (2, 3, 4, 5, 6, 7)]
    with Robot((2, 3, 4), (5, 6, 7)) as robot:
        left_motor, right_motor = robot.all
        assert isinstance(left_motor, Motor)
        assert left_motor.forward_device.pin is pins[0]
        assert isinstance(left_motor.forward_device, PWMOutputDevice)
        assert left_motor.backward_device.pin is pins[1]
        assert isinstance(left_motor.forward_device, PWMOutputDevice)
        assert left_motor.enable_device.pin is pins[2]
        assert isinstance(left_motor.enable_device, DigitalOutputDevice)
        assert isinstance(right_motor, Motor)
        assert right_motor.forward_device.pin is pins[3]
        assert isinstance(right_motor.forward_device, PWMOutputDevice)
        assert right_motor.backward_device.pin is pins[4]
        assert isinstance(right_motor.backward_device, PWMOutputDevice)
        assert right_motor.enable_device.pin is pins[5]
        assert isinstance(right_motor.enable_device, DigitalOutputDevice)

def test_enable_pin_motor_robot_nopwm(mock_factory):
    pins = [mock_factory.pin(n) for n in (2, 3, 4, 5, 6, 7)]
    with Robot((2, 3, 4), (5, 6, 7), pwm=False) as robot:
        left_motor, right_motor = robot.all
        assert isinstance(left_motor, Motor)
        assert left_motor.forward_device.pin is pins[0]
        assert isinstance(left_motor.forward_device, DigitalOutputDevice)
        assert left_motor.backward_device.pin is pins[1]
        assert isinstance(left_motor.forward_device, DigitalOutputDevice)
        assert left_motor.enable_device.pin is pins[2]
        assert isinstance(left_motor.enable_device, OutputDevice)
        assert isinstance(right_motor, Motor)
        assert right_motor.forward_device.pin is pins[3]
        assert isinstance(right_motor.forward_device, DigitalOutputDevice)
        assert right_motor.backward_device.pin is pins[4]
        assert isinstance(right_motor.backward_device, DigitalOutputDevice)
        assert right_motor.enable_device.pin is pins[5]
        assert isinstance(right_motor.enable_device, DigitalOutputDevice)

def test_phaseenable_robot_bad_init(mock_factory):
    with pytest.raises(GPIOPinMissing):
        PhaseEnableRobot()
    with pytest.raises(GPIOPinMissing):
        PhaseEnableRobot(2)
    with pytest.raises(GPIOPinMissing):
        PhaseEnableRobot(2, 3)
    with pytest.raises(GPIOPinMissing):
        PhaseEnableRobot(2, 3, 4)
    with pytest.raises(GPIOPinMissing):
        PhaseEnableRobot(2, 3, 4, 5)
    with pytest.raises(GPIOPinMissing):
        PhaseEnableRobot(2, 3, 4, 5, 6, 7)
    with pytest.raises(GPIOPinMissing):
        PhaseEnableRobot((2, 3))
    with pytest.raises(GPIOPinMissing):
        PhaseEnableRobot((2, 3), 4)

def test_phaseenable_robot(mock_factory, pwm):
    pins = [mock_factory.pin(n) for n in (5, 12, 6, 13)]
    with PhaseEnableRobot((5, 12), (6, 13)) as robot:
        assert repr(robot).startswith('<gpiozero.PhaseEnableRobot object')
        assert (
            [device.pin for device in robot.left_motor] +
            [device.pin for device in robot.right_motor]) == pins
        assert robot.value == (0, 0)
        robot.forward()
        assert [pin.state for pin in pins] == [0, 1, 0, 1]
        assert robot.value == (1, 1)
        robot.backward()
        assert [pin.state for pin in pins] == [1, 1, 1, 1]
        assert robot.value == (-1, -1)
        robot.forward(0.5)
        assert [pin.state for pin in pins] == [0, 0.5, 0, 0.5]
        assert robot.value == (0.5, 0.5)
        robot.left()
        assert [pin.state for pin in pins] == [1, 1, 0, 1]
        assert robot.value == (-1, 1)
        robot.right()
        assert [pin.state for pin in pins] == [0, 1, 1, 1]
        assert robot.value == (1, -1)
        robot.reverse()
        assert [pin.state for pin in pins] == [1, 1, 0, 1]
        assert robot.value == (-1, 1)
        robot.stop()
        assert [pin.state for pin in pins][1::2] == [0, 0]
        assert robot.value == (0, 0)
        robot.value = (-1, -1)
        assert robot.value == (-1, -1)
        robot.value = (0.5, 1)
        assert robot.value == (0.5, 1)
        robot.value = (0, -0.5)
        assert robot.value == (0, -0.5)

def test_energenie_bad_init(mock_factory):
    with pytest.raises(ValueError):
        Energenie()
    with pytest.raises(ValueError):
        Energenie(0)
    with pytest.raises(ValueError):
        Energenie(5)

def test_energenie(mock_factory):
    pins = [mock_factory.pin(n) for n in (17, 22, 23, 27, 24, 25)]
    with Energenie(1, initial_value=True) as device1, \
         Energenie(2, initial_value=False) as device2, \
         Energenie(3, initial_value=None) as device3:
        assert repr(device1) == '<gpiozero.Energenie object on socket 1>'
        assert repr(device2) == '<gpiozero.Energenie object on socket 2>'
        assert repr(device3) == '<gpiozero.Energenie object on socket 3>'
        assert device1.value
        assert not device2.value
        assert device3.value is None
        assert device1.socket == 1
        assert device2.socket == 2
        assert device3.socket == 3
        [pin.clear_states() for pin in pins]
        device1.on()
        assert device1.value
        pins[0].assert_states_and_times([(0.0, False), (0.0, True)])
        pins[1].assert_states_and_times([(0.0, True), (0.0, True)])
        pins[2].assert_states_and_times([(0.0, True), (0.0, True)])
        pins[3].assert_states_and_times([(0.0, False), (0.0, True)])
        pins[4].assert_states_and_times([(0.0, False)])
        pins[5].assert_states_and_times([(0.0, False), (0.1, True), (0.25, False)])
        [pin.clear_states() for pin in pins]
        device2.on()
        assert device2.value
        pins[0].assert_states_and_times([(0.0, True), (0.0, False)])
        pins[1].assert_states_and_times([(0.0, True), (0.0, True)])
        pins[2].assert_states_and_times([(0.0, True), (0.0, True)])
        pins[3].assert_states_and_times([(0.0, True), (0.0, True)])
        pins[4].assert_states_and_times([(0.0, False)])
        pins[5].assert_states_and_times([(0.0, False), (0.1, True), (0.25, False)])
        [pin.clear_states() for pin in pins]
        device3.on()
        assert device3.value
        pins[0].assert_states_and_times([(0.0, False), (0.0, True)])
        pins[1].assert_states_and_times([(0.0, True), (0.0, False)])
        pins[2].assert_states_and_times([(0.0, True), (0.0, False)])
        pins[3].assert_states_and_times([(0.0, True), (0.0, False)])
        pins[4].assert_states_and_times([(0.0, False)])
        pins[5].assert_states_and_times([(0.0, False), (0.1, True), (0.25, False)])

        device3.value = False
        assert not device3.value
        device3.value = True
        assert device3.value
        with pytest.raises(TypeError):
            device3.value = None

        device1.close()
        assert repr(device1) == '<gpiozero.Energenie object closed>'

def test_statuszero_init(mock_factory):
    with StatusZero() as sz:
        assert repr(sz).startswith('<gpiozero.StatusZero object')
        assert sz.namedtuple._fields == ('one', 'two', 'three')
    with StatusZero('a') as sz:
        assert sz.namedtuple._fields == ('a',)
    with StatusZero('a', 'b') as sz:
        assert sz.namedtuple._fields == ('a', 'b')
    with StatusZero('a', 'b', 'c') as sz:
        assert sz.namedtuple._fields == ('a', 'b', 'c')
    with pytest.raises(ValueError):
        StatusZero('a', 'b', 'c', 'd')
    with pytest.raises(ValueError):
        StatusZero('0')
    with pytest.raises(ValueError):
        StatusZero('foo', 'hello world')
    with pytest.raises(ValueError):
        StatusZero('foo', 'foo')

def test_statuszero(mock_factory):
    with StatusZero() as sz:
        assert isinstance(sz.one, LEDBoard)
        assert isinstance(sz.two, LEDBoard)
        assert isinstance(sz.three, LEDBoard)
        assert isinstance(sz.one.red, LED)
        assert isinstance(sz.one.green, LED)
        assert sz.value == ((False, False), (False, False), (False, False))
        sz.on()
        assert sz.value == ((True, True), (True, True), (True, True))
        sz.one.green.off()
        assert sz.one.value == (True, False)

def test_statuszero_kwargs(mock_factory, pwm):
    with StatusZero(pwm=True, initial_value=True) as sz:
        assert isinstance(sz.one, LEDBoard)
        assert isinstance(sz.two, LEDBoard)
        assert isinstance(sz.three, LEDBoard)
        assert isinstance(sz.one.red, PWMLED)
        assert isinstance(sz.one.green, PWMLED)
        assert sz.value == ((1, 1), (1, 1), (1, 1))
        sz.off()
        assert sz.value == ((0, 0), (0, 0), (0, 0))

def test_statuszero_named(mock_factory):
    with StatusZero('a') as sz:
        assert isinstance(sz.a, LEDBoard)
        assert isinstance(sz.a.red, LED)
        with pytest.raises(AttributeError):
            sz.one

def test_statusboard_init(mock_factory):
    with StatusBoard() as sb:
        assert repr(sb).startswith('<gpiozero.StatusBoard object')
        assert sb.namedtuple._fields == ('one', 'two', 'three', 'four', 'five')
    with StatusBoard('a') as sb:
        assert sb.namedtuple._fields == ('a',)
    with StatusBoard('a', 'b') as sb:
        assert sb.namedtuple._fields == ('a', 'b',)
    with StatusBoard('a', 'b', 'c', 'd', 'e') as sb:
        assert sb.namedtuple._fields == ('a', 'b', 'c', 'd', 'e')
    with pytest.raises(ValueError):
        StatusBoard('a', 'b', 'c', 'd', 'e', 'f')
    with pytest.raises(ValueError):
        StatusBoard('0')
    with pytest.raises(ValueError):
        StatusBoard('foo', 'hello world')
    with pytest.raises(ValueError):
        StatusBoard('foo', 'foo')

def test_statusboard(mock_factory):
    with StatusBoard() as sb:
        assert isinstance(sb.one, CompositeOutputDevice)
        assert isinstance(sb.two, CompositeOutputDevice)
        assert isinstance(sb.five, CompositeOutputDevice)
        assert isinstance(sb.one.button, Button)
        assert isinstance(sb.one.lights, LEDBoard)
        assert isinstance(sb.one.lights.red, LED)
        assert isinstance(sb.one.lights.green, LED)
        assert sb.value == ((False, (False, False)), (False, (False, False)),
                            (False, (False, False)), (False, (False, False)),
                            (False, (False, False)))
        sb.on()
        assert sb.value == ((False, (True, True)), (False, (True, True)),
                            (False, (True, True)), (False, (True, True)),
                            (False, (True, True)))
        sb.one.lights.green.off()
        assert sb.one.value == (False, (True, False))

def test_statusboard_kwargs(mock_factory, pwm):
    with StatusBoard(pwm=True, initial_value=True) as sb:
        assert isinstance(sb.one, CompositeOutputDevice)
        assert isinstance(sb.two, CompositeOutputDevice)
        assert isinstance(sb.five, CompositeOutputDevice)
        assert isinstance(sb.one.button, Button)
        assert isinstance(sb.one.lights, LEDBoard)
        assert isinstance(sb.one.lights.red, PWMLED)
        assert isinstance(sb.one.lights.green, PWMLED)
        assert sb.value == ((False, (1, 1)), (False, (1, 1)), (False, (1, 1)),
                           (False, (1, 1)), (False, (1, 1)))
        sb.off()
        assert sb.value == ((False, (0, 0)), (False, (0, 0)), (False, (0, 0)),
                            (False, (0, 0)), (False, (0, 0)))

def test_statusboard_named(mock_factory):
    with StatusBoard('a') as sb:
        assert isinstance(sb.a, CompositeOutputDevice)
        assert isinstance(sb.a.button, Button)
        assert isinstance(sb.a.lights, LEDBoard)
        assert isinstance(sb.a.lights.red, LED)
        with pytest.raises(AttributeError):
            sb.one

def test_pumpkin_pi(mock_factory):
    pins = [mock_factory.pin(n)
            for n in (12, 6, 18, 17, 16, 13, 24, 19, 20, 21, 22, 23)]
    with PumpkinPi() as board:
        assert repr(board).startswith('<gpiozero.PumpkinPi object')
        assert [device.pin for device in board.leds] == pins
        assert isinstance(board.sides.left, LEDBoard)
        assert isinstance(board.sides.right, LEDBoard)
        assert isinstance(board.sides, LEDBoard)
        assert isinstance(board.eyes, LEDBoard)
        board.off()
        assert board.value == (
            (False, False), (
                (False, False, False, False, False),
                (False, False, False, False, False)
            )
        )
        board.eyes.on()
        assert board.value == (
            (True, True), (
                (False, False, False, False, False),
                (False, False, False, False, False)
            )
        )
        board.sides.left.middle.on()
        assert board.value == (
            (True, True), (
                (False, False, True, False, False),
                (False, False, False, False, False)
            )
        )
        board.sides.right.midtop.on()
        assert board.value == (
            (True, True), (
                (False, False, True, False, False),
                (False, False, False, True, False)
            )
        )
        board.toggle()
        assert board.value == (
            (False, False), (
                (True, True, False, True, True),
                (True, True, True, False, True)
            )
        )

def test_pumpkin_pi_initial_value(mock_factory):
    with PumpkinPi() as board:
        assert not any(device.pin.state for device in board.leds)
    with PumpkinPi(initial_value=False) as board:
        assert not any(device.pin.state for device in board.leds)
    with PumpkinPi(initial_value=True) as board:
        assert all(device.pin.state for device in board.leds)
    with PumpkinPi(initial_value=0.5) as board:
        assert all(device.pin.state for device in board.leds)

def test_pumpkin_pi_initial_value_pwm(mock_factory, pwm):
    with PumpkinPi(pwm=True, initial_value=0.5) as board:
        assert all(device.pin.state == 0.5 for device in board.leds)

def test_jamhat(mock_factory, pwm):
    with JamHat() as jh:
        assert repr(jh).startswith('<gpiozero.JamHat object')
        assert isinstance(jh.lights_1, LEDBoard)
        assert isinstance(jh.lights_2, LEDBoard)
        assert isinstance(jh.button_1, Button)
        assert isinstance(jh.button_2, Button)
        assert isinstance(jh.buzzer, TonalBuzzer)
        assert isinstance(jh.lights_1.red, LED)
        assert jh.value == (
            (False, False, False),  # lights_1
            (False, False, False),  # lights_2
            False, False,           # buttons
            None                    # buzzer
        )
        jh.on()
        assert jh.value == (
            (True, True, True),  # lights_1
            (True, True, True),  # lights_2
            False, False,        # buttons
            0                    # buzzer
        )
        jh.buzzer.play('A5')
        jh.button_1.pin.drive_high()
        jh.button_2.pin.drive_high()
        assert jh.value == (
            (True, True, True),  # lights_1
            (True, True, True),  # lights_2
            True, True,          # buttons
            1                    # buzzer
        )
        jh.off()
        assert jh.value == (
            (False, False, False),  # lights_1
            (False, False, False),  # lights_2
            True, True,             # buttons
            None                    # buzzer
        )

def test_jamhat_pwm(mock_factory, pwm):
    with JamHat(pwm=True) as jh:
        assert isinstance(jh.lights_1, LEDBoard)
        assert isinstance(jh.lights_1.red, PWMLED)
        assert isinstance(jh.lights_2, LEDBoard)
        assert isinstance(jh.button_1, Button)
        assert isinstance(jh.button_2, Button)
        assert isinstance(jh.buzzer, TonalBuzzer)
        assert jh.value == (
            (0, 0, 0),              # lights_1
            (0, 0, 0),              # lights_2
            False, False,           # buttons
            None                    # buzzer
        )
        jh.on()
        assert jh.value == (
            (1, 1, 1),           # lights_1
            (1, 1, 1),           # lights_2
            False, False,        # buttons
            0                    # buzzer
        )
        jh.off()
        assert jh.value == (
            (0, 0, 0),              # lights_1
            (0, 0, 0),              # lights_2
            False, False,           # buttons
            None                    # buzzer
        )

def test_pibrella(mock_factory, pwm):
    with Pibrella() as pb:
        assert repr(pb).startswith('<gpiozero.Pibrella object')
        assert isinstance(pb.lights, TrafficLights)
        assert isinstance(pb.lights.red, LED)
        assert isinstance(pb.button, Button)
        assert isinstance(pb.buzzer, TonalBuzzer)
        assert pb.inputs == ('BOARD21', 'BOARD26', 'BOARD24', 'BOARD19')
        assert pb.outputs == ('BOARD15', 'BOARD16', 'BOARD18', 'BOARD22')
        assert pb.value == (
            (0, 0, 0),  # lights
            0,          # button
            None        # buzzer
        )
        pb.on()
        assert pb.value == (
            (1, 1, 1),  # lights
            0,          # button
            0           # buzzer
        )
        pb.buzzer.play('A5')
        pb.button.pin.drive_high()
        assert pb.value == (
            (1, 1, 1),  # lights
            1,          # button
            1           # buzzer
        )
        pb.off()
        assert pb.value == (
            (0, 0, 0),  # lights
            1,          # button
            None        # buzzer
        )

def test_pibrella_pwm(mock_factory, pwm):
    with Pibrella(pwm=True) as pb:
        assert isinstance(pb.lights, TrafficLights)
        assert isinstance(pb.lights.red, PWMLED)
        assert isinstance(pb.button, Button)
        assert isinstance(pb.buzzer, TonalBuzzer)
        assert pb.inputs == ('BOARD21', 'BOARD26', 'BOARD24', 'BOARD19')
        assert pb.outputs == ('BOARD15', 'BOARD16', 'BOARD18', 'BOARD22')
        assert pb.value == (
            (0, 0, 0),  # lights
            0,          # button
            None        # buzzer
        )
        pb.on()
        assert pb.value == (
            (1, 1, 1),  # lights
            0,          # button
            0           # buzzer
        )
        pb.off()
        assert pb.value == (
            (0, 0, 0),  # lights
            0,          # button
            None        # buzzer
        )

def test_pibrella_pins(mock_factory, pwm):
    with Pibrella() as pb:
        with LED(pb.outputs.e) as led:
            assert isinstance(led, LED)
            assert led.pin.number == 22
        with Button(pb.inputs.a) as btn:
            assert btn.pin.number == 9

def test_led_char_display_init(mock_factory):
    pins = [mock_factory.pin(i) for i in range(28)]
    with pytest.raises(PinInvalidPin):
        LEDCharDisplay(*range(28))
    with LEDCollection(*range(10)) as collection:
        with pytest.raises(PinInvalidPin):
            LEDCharDisplay(collection, *range(4))
    with LEDCharDisplay(*range(4)) as char:
        assert len(char.font) == 1
        assert ' ' in char.font
    default_font = load_font_7seg('gpiozero/fonts/7seg.txt')
    with LEDCharDisplay(*range(10, 17), font=default_font) as char:
        assert char.font == default_font
        char.value = '2'
    with LEDCharDisplay(*range(10, 17), initial_value=None) as char:
        assert char.value == '2'

def test_led_char_display_value(mock_factory):
    pins = [mock_factory.pin(i) for i in range(4, 11)]
    dp_pin = mock_factory.pin(11)
    with LEDCharDisplay(*range(4, 11), dp=11) as char:
        assert char.value == ' '
        assert not any(dev.value for dev in char)
        char.value = '8'
        assert all(dev.value for dev in char if dev is not char.dp)
        assert char.value == '8'
        char.value = '8.'
        assert all(dev.value for dev in char)
        assert char.value == '8.'

def test_led_char_display_font(mock_factory):
    pins = [mock_factory.pin(i) for i in range(4, 11)]
    dp_pin = mock_factory.pin(11)
    with LEDCharDisplay(*range(4, 11), dp=11) as char:
        char.value = '5'
        del char.font['5']
        assert char.value == 'S'
        before_states = [int(p.state) for p in pins]
        char.font['S'] = (0, 0, 1, 1, 0, 1, 1)
        after_states = [int(p.state) for p in pins]
        assert before_states == after_states
        assert char.value is None
        char.font['b'] = char.font['B']
        char.value = 'B'
        assert char.value == 'B'  # always uses first definition
        del char.font['b']
        assert char.value == 'B'  # doesn't damage reverse map

    with LEDCharDisplay(*range(4, 11)) as char:
        char.value = '0'
        font = {
            i: char.font[chr(ord('0') + i)]
            for i in range(10)
        }
        char.font = font
        assert char.value == 0
        assert repr(char.font) == """LEDCharFont({
    0: (1, 1, 1, 1, 1, 1, 0),
    1: (0, 1, 1, 0, 0, 0, 0),
    2: (1, 1, 0, 1, 1, 0, 1),
    3: (1, 1, 1, 1, 0, 0, 1),
    4: (0, 1, 1, 0, 0, 1, 1),
    5: (1, 0, 1, 1, 0, 1, 1),
    6: (1, 0, 1, 1, 1, 1, 1),
    7: (1, 1, 1, 0, 0, 0, 0),
    8: (1, 1, 1, 1, 1, 1, 1),
    9: (1, 1, 1, 1, 0, 1, 1),
})"""

def test_led_multichar_display_init(mock_factory):
    pins = [mock_factory.pin(i) for i in range(10, 22)]
    with pytest.raises(ValueError):
        LEDMultiCharDisplay(object())
    with LEDCharDisplay(*range(10, 17), dp=17) as char:
        with LEDMultiCharDisplay(char, *range(18, 22)) as multichar:
            assert multichar.value == (' ',) * 4
    with LEDCharDisplay(*range(10, 17), dp=17) as char:
        with LEDMultiCharDisplay(char, *range(18, 22), initial_value='GPIO') as multichar:
            assert multichar.value == ('G', 'P', 'I', 'O')
    with LEDCharDisplay(*range(10, 17), dp=17) as char:
        with LEDMultiCharDisplay(char, *range(18, 22), pin_factory=mock_factory) as multichar:
            assert multichar.pin_factory is char.pin_factory

def test_led_multichar_display_plex_delay(mock_factory):
    pins = [mock_factory.pin(i) for i in range(10, 22)]
    with LEDCharDisplay(*range(10, 17), dp=17) as char:
        with LEDMultiCharDisplay(char, *range(18, 22)) as multichar:
            multichar.plex_delay = 0.1
            assert multichar.plex_delay == 0.1
            with pytest.raises(ValueError):
                multichar.plex_delay = -1

def test_led_multichar_display_value(mock_factory):
    pins = [mock_factory.pin(i) for i in range(10, 22)]
    with LEDCharDisplay(*range(10, 17), dp=17) as char:
        with LEDMultiCharDisplay(char, *range(18, 22)) as multichar:
            with pytest.raises(ValueError):
                multichar.value = 'FOOBARBAZ'
            stop = Event()
            char_set = Event()
            char_checked = Event()
            def my_set():
                stop.set()
            def my_wait(timeout):
                char_set.set()
                char_checked.wait(0.1)
                char_checked.clear()
                return stop.wait(0)
            with mock.patch('gpiozero.threads.Event') as event_mock:
                event_mock().wait.side_effect = my_wait
                event_mock().set.side_effect = my_set
                multichar.value = 'GPIO'
                states = {
                    ('G', (1, 0, 0, 0)),
                    ('P', (0, 1, 0, 0)),
                    ('I', (0, 0, 1, 0)),
                    ('O', (0, 0, 0, 1)),
                }
                for i in range(4):
                    assert char_set.wait(0.1)
                    char_set.clear()
                    try:
                        states.remove((multichar.char.value, multichar.plex.value))
                    except KeyError:
                        assert False, 'invalid state'
                    if not states:
                        multichar.value = 'GG'
                    char_checked.set()
                assert not states
                # Static value requires no transitions despite still having
                # active characters
                assert not multichar._plex_thread



from travispy import TravisPy
from gpiozero import LED
from gpiozero.tools import negated
from time import sleep
from signal import pause

def build_passed(repo):
    t = TravisPy()
    r = t.repo(repo)
    while True:
        yield r.last_build_state == 'passed'

red = LED(12)
green = LED(16)

green.source = build_passed('gpiozero/gpiozero')
green.source_delay = 60 * 5  # check every 5 minutes
red.source = negated(green)

pause()



from gpiozero import PingServer, LEDBoard
from gpiozero.tools import negated
from signal import pause

status = LEDBoard(
    mum=LEDBoard(red=14, green=15),
    dad=LEDBoard(red=17, green=18),
    alice=LEDBoard(red=21, green=22)
)

statuses = {
    PingServer('192.168.1.5'): status.mum,
    PingServer('192.168.1.6'): status.dad,
    PingServer('192.168.1.7'): status.alice,
}

for server, leds in statuses.items():
    leds.green.source = server
    leds.green.source_delay = 60
    leds.red.source = negated(leds.green)

pause()



from gpiozero import TrafficLights
from time import sleep

lights = TrafficLights(2, 3, 4)

lights.green.on()

while True:
    sleep(10)
    lights.green.off()
    lights.amber.on()
    sleep(1)
    lights.amber.off()
    lights.red.on()
    sleep(10)
    lights.amber.on()
    sleep(1)
    lights.green.on()
    lights.amber.off()
    lights.red.off()



# GPIO Zero: a library for controlling the Raspberry Pi's GPIO pins
#
# Copyright (c) 2017-2019 Dave Jones <dave@waveform.org.uk>
# Copyright (c) 2016 Stewart <stewart@adcock.org.uk>
#
# SPDX-License-Identifier: BSD-3-Clause

from __future__ import (
    unicode_literals,
    absolute_import,
    print_function,
    division,
    )
str = type('')


import os

import pytest

from gpiozerocli.pinout import main


def test_args_incorrect():
    with pytest.raises(SystemExit) as ex:
        main(['pinout', '--nonexistentarg'])

def test_args_color():
    args = main.parser.parse_args([])
    assert args.color is None
    args = main.parser.parse_args(['--color'])
    assert args.color is True
    args = main.parser.parse_args(['--monochrome'])
    assert args.color is False

def test_args_revision():
    args = main.parser.parse_args(['--revision', '000d'])
    assert args.revision == '000d'

def test_help(capsys):
    with pytest.raises(SystemExit) as ex:
        main(['pinout', '--help'])
    out, err = capsys.readouterr()
    assert 'GPIO pin-out' in out

def test_execution(capsys, no_default_factory):
    os.environ['GPIOZERO_PIN_FACTORY'] = 'mock'
    assert main([]) == 0



from gpiozero import Robot
from bluedot import BlueDot
from signal import pause

def pos_to_values(x, y):
    left = y if x > 0 else y + x
    right = y if x < 0 else y - x
    return (clamped(left), clamped(right))

def clamped(v):
    return max(-1, min(1, v))

def drive():
    while True:
        if bd.is_pressed:
            x, y = bd.position.x, bd.position.y
            yield pos_to_values(x, y)
        else:
            yield (0, 0)

robot = Robot(left=(4, 14), right=(17, 18))
bd = BlueDot()

robot.source = drive()

pause()



from gpiozero import Button, LED
from gpiozero.pins.pigpio import PiGPIOFactory
from signal import pause

factory = PiGPIOFactory(host='192.168.1.3')

button = Button(2)
led = LED(17, pin_factory=factory)

led.source = button

pause()



from gpiozero import Button, LED
from gpiozero.pins.pigpio import PiGPIOFactory
from gpiozero.tools import all_values
from signal import pause

factory3 = PiGPIOFactory(host='192.168.1.3')
factory4 = PiGPIOFactory(host='192.168.1.4')

led = LED(17)
button_1 = Button(17, pin_factory=factory3)
button_2 = Button(17, pin_factory=factory4)

led.source = all_values(button_1, button_2)

pause()



# vim: set fileencoding=utf-8:
#
# GPIO Zero: a library for controlling the Raspberry Pi's GPIO pins
#
# Copyright (c) 2016-2021 Dave Jones <dave@waveform.org.uk>
# Copyright (c) 2019-2020 Ben Nuttall <ben@bennuttall.com>
# Copyright (c) 2019 Kosovan Sofiia <sofiia.kosovan@gmail.com>
# Copyright (c) 2016 Andrew Scheller <github@loowis.durge.org>
#
# SPDX-License-Identifier: BSD-3-Clause

from __future__ import (
    unicode_literals,
    print_function,
    absolute_import,
    division,
)
str = type('')


class GPIOZeroError(Exception):
    "Base class for all exceptions in GPIO Zero"

class DeviceClosed(GPIOZeroError):
    "Error raised when an operation is attempted on a closed device"

class BadEventHandler(GPIOZeroError, ValueError):
    "Error raised when an event handler with an incompatible prototype is specified"

class BadWaitTime(GPIOZeroError, ValueError):
    "Error raised when an invalid wait time is specified"

class BadQueueLen(GPIOZeroError, ValueError):
    "Error raised when non-positive queue length is specified"

class BadPinFactory(GPIOZeroError, ImportError):
    "Error raised when an unknown pin factory name is specified"

class ZombieThread(GPIOZeroError, RuntimeError):
    "Error raised when a thread fails to die within a given timeout"

class CompositeDeviceError(GPIOZeroError):
    "Base class for errors specific to the CompositeDevice hierarchy"

class CompositeDeviceBadName(CompositeDeviceError, ValueError):
    "Error raised when a composite device is constructed with a reserved name"

class CompositeDeviceBadOrder(CompositeDeviceError, ValueError):
    "Error raised when a composite device is constructed with an incomplete order"

class CompositeDeviceBadDevice(CompositeDeviceError, ValueError):
    "Error raised when a composite device is constructed with an object that doesn't inherit from :class:`Device`"

class EnergenieSocketMissing(CompositeDeviceError, ValueError):
    "Error raised when socket number is not specified"

class EnergenieBadSocket(CompositeDeviceError, ValueError):
    "Error raised when an invalid socket number is passed to :class:`Energenie`"

class SPIError(GPIOZeroError):
    "Base class for errors related to the SPI implementation"

class SPIBadArgs(SPIError, ValueError):
    "Error raised when invalid arguments are given while constructing :class:`SPIDevice`"

class SPIBadChannel(SPIError, ValueError):
    "Error raised when an invalid channel is given to an :class:`AnalogInputDevice`"

class SPIFixedClockMode(SPIError, AttributeError):
    "Error raised when the SPI clock mode cannot be changed"

class SPIInvalidClockMode(SPIError, ValueError):
    "Error raised when an invalid clock mode is given to an SPI implementation"

class SPIFixedBitOrder(SPIError, AttributeError):
    "Error raised when the SPI bit-endianness cannot be changed"

class SPIFixedSelect(SPIError, AttributeError):
    "Error raised when the SPI select polarity cannot be changed"

class SPIFixedWordSize(SPIError, AttributeError):
    "Error raised when the number of bits per word cannot be changed"

class SPIFixedRate(SPIError, AttributeError):
    "Error raised when the baud-rate of the interface cannot be changed"

class SPIInvalidWordSize(SPIError, ValueError):
    "Error raised when an invalid (out of range) number of bits per word is specified"

class GPIODeviceError(GPIOZeroError):
    "Base class for errors specific to the GPIODevice hierarchy"

class GPIODeviceClosed(GPIODeviceError, DeviceClosed):
    "Deprecated descendent of :exc:`DeviceClosed`"

class GPIOPinInUse(GPIODeviceError):
    "Error raised when attempting to use a pin already in use by another device"

class GPIOPinMissing(GPIODeviceError, ValueError):
    "Error raised when a pin specification is not given"

class InputDeviceError(GPIODeviceError):
    "Base class for errors specific to the InputDevice hierarchy"

class OutputDeviceError(GPIODeviceError):
    "Base class for errors specified to the OutputDevice hierarchy"

class OutputDeviceBadValue(OutputDeviceError, ValueError):
    "Error raised when ``value`` is set to an invalid value"

class PinError(GPIOZeroError):
    "Base class for errors related to pin implementations"

class PinInvalidFunction(PinError, ValueError):
    "Error raised when attempting to change the function of a pin to an invalid value"

class PinInvalidState(PinError, ValueError):
    "Error raised when attempting to assign an invalid state to a pin"

class PinInvalidPull(PinError, ValueError):
    "Error raised when attempting to assign an invalid pull-up to a pin"

class PinInvalidEdges(PinError, ValueError):
    "Error raised when attempting to assign an invalid edge detection to a pin"

class PinInvalidBounce(PinError, ValueError):
    "Error raised when attempting to assign an invalid bounce time to a pin"

class PinSetInput(PinError, AttributeError):
    "Error raised when attempting to set a read-only pin"

class PinFixedPull(PinError, AttributeError):
    "Error raised when attempting to set the pull of a pin with fixed pull-up"

class PinEdgeDetectUnsupported(PinError, AttributeError):
    "Error raised when attempting to use edge detection on unsupported pins"

class PinUnsupported(PinError, NotImplementedError):
    "Error raised when attempting to obtain a pin interface on unsupported pins"

class PinSPIUnsupported(PinError, NotImplementedError):
    "Error raised when attempting to obtain an SPI interface on unsupported pins"

class PinPWMError(PinError):
    "Base class for errors related to PWM implementations"

class PinPWMUnsupported(PinPWMError, AttributeError):
    "Error raised when attempting to activate PWM on unsupported pins"

class PinPWMFixedValue(PinPWMError, AttributeError):
    "Error raised when attempting to initialize PWM on an input pin"

class PinUnknownPi(PinError, RuntimeError):
    "Error raised when gpiozero doesn't recognize a revision of the Pi"

class PinMultiplePins(PinError, RuntimeError):
    "Error raised when multiple pins support the requested function"

class PinNoPins(PinError, RuntimeError):
    "Error raised when no pins support the requested function"

class PinInvalidPin(PinError, ValueError):
    "Error raised when an invalid pin specification is provided"

class GPIOZeroWarning(Warning):
    "Base class for all warnings in GPIO Zero"

class DistanceSensorNoEcho(GPIOZeroWarning):
    "Warning raised when the distance sensor sees no echo at all"

class SPIWarning(GPIOZeroWarning):
    "Base class for warnings related to the SPI implementation"

class SPISoftwareFallback(SPIWarning):
    "Warning raised when falling back to the SPI software implementation"

class PWMWarning(GPIOZeroWarning):
    "Base class for PWM warnings"

class PWMSoftwareFallback(PWMWarning):
    "Warning raised when falling back to the PWM software implementation"

class PinWarning(GPIOZeroWarning):
    "Base class for warnings related to pin implementations"

class PinFactoryFallback(PinWarning):
    "Warning raised when a default pin factory fails to load and a fallback is tried"

class NativePinFactoryFallback(PinWarning):
    "Warning raised when all other default pin factories fail to load and NativeFactory is used"

class PinNonPhysical(PinWarning):
    "Warning raised when a non-physical pin is specified in a constructor"

class ThresholdOutOfRange(GPIOZeroWarning):
    "Warning raised when a threshold is out of range specified by min and max values"

class CallbackSetToNone(GPIOZeroWarning):
    "Warning raised when a callback is set to None when its previous value was None"

class AmbiguousTone(GPIOZeroWarning):
    "Warning raised when a Tone is constructed with an ambiguous number"



"A simple interface to GPIO devices with Raspberry Pi."

import io
import os
import sys
import errno
from setuptools import setup, find_packages

if sys.version_info[0] == 2:
    if not sys.version_info >= (2, 7):
        raise ValueError('This package requires Python 2.7 or above')
elif sys.version_info[0] == 3:
    if not sys.version_info >= (3, 2):
        raise ValueError('This package requires Python 3.2 or above')
else:
    raise ValueError('Unrecognized major version of Python')

HERE = os.path.abspath(os.path.dirname(__file__))

# Workaround <http://www.eby-sarna.com/pipermail/peak/2010-May/003357.html>
try:
    import multiprocessing
except ImportError:
    pass

__project__      = 'gpiozero'
__version__      = '1.6.1'
__author__       = 'Ben Nuttall'
__author_email__ = 'ben@bennuttall.com'
__url__          = 'https://github.com/gpiozero/gpiozero'
__platforms__    = 'ALL'

__classifiers__ = [
    "Development Status :: 5 - Production/Stable",
    "Intended Audience :: Education",
    "Intended Audience :: Developers",
    "Topic :: Education",
    "Topic :: System :: Hardware",
    "License :: OSI Approved :: BSD License",
    "Programming Language :: Python :: 2",
    "Programming Language :: Python :: 2.7",
    "Programming Language :: Python :: 3",
    "Programming Language :: Python :: 3.5",
    "Programming Language :: Python :: 3.6",
    "Programming Language :: Python :: 3.7",
    "Programming Language :: Python :: 3.8",
    "Programming Language :: Python :: 3.9",
    "Programming Language :: Python :: Implementation :: PyPy",
]

__keywords__ = [
    'raspberrypi',
    'gpio',
]

__requires__ = [
    'colorzero',
]

__extra_requires__ = {
    'doc':   ['sphinx', 'sphinx_rtd_theme'],
    'test':  ['pytest', 'coverage', 'mock'],
}

if sys.version_info[:2] == (3, 2):
    # Particular versions are required for Python 3.2 compatibility
    __extra_requires__['doc'].extend([
        'Jinja2<2.7',
        'MarkupSafe<0.16',
        ])
    __extra_requires__['test'][0] = 'pytest<3.0dev'
    __extra_requires__['test'][1] = 'coverage<4.0dev'
elif sys.version_info[:2] == (3, 3):
    __extra_requires__['test'][0] = 'pytest<3.3dev'
elif sys.version_info[:2] == (3, 4):
    __extra_requires__['test'][0] = 'pytest<5.0dev'

try:
    # If we're executing on a Raspberry Pi, install all GPIO libraries for
    # testing (except RPIO which doesn't work on the multi-core models yet)
    with io.open('/proc/device-tree/model', 'r') as f:
        if f.read().startswith('Raspberry Pi'):
            __extra_requires__['test'].append('RPi.GPIO')
            __extra_requires__['test'].append('pigpio')
except IOError as e:
    if e.errno != errno.ENOENT:
        raise

__entry_points__ = {
    'gpiozero_pin_factories': [
        'pigpio  = gpiozero.pins.pigpio:PiGPIOFactory',
        'lgpio   = gpiozero.pins.lgpio:LGPIOFactory',
        'rpigpio = gpiozero.pins.rpigpio:RPiGPIOFactory',
        'rpio    = gpiozero.pins.rpio:RPIOFactory',
        'native  = gpiozero.pins.native:NativeFactory',
        'mock    = gpiozero.pins.mock:MockFactory',
        # Backwards compatible names
        'PiGPIOPin  = gpiozero.pins.pigpio:PiGPIOFactory',
        'RPiGPIOPin = gpiozero.pins.rpigpio:RPiGPIOFactory',
        'RPIOPin    = gpiozero.pins.rpio:RPIOFactory',
        'NativePin  = gpiozero.pins.native:NativeFactory',
    ],
    'gpiozero_mock_pin_classes': [
        'mockpin          = gpiozero.pins.mock:MockPin',
        'mockpwmpin       = gpiozero.pins.mock:MockPWMPin',
        'mockchargingpin  = gpiozero.pins.mock:MockChargingPin',
        'mocktriggerpin   = gpiozero.pins.mock:MockTriggerPin',
    ],
    'console_scripts': [
        'pinout = gpiozerocli.pinout:main',
    ]
}


def main():
    import io
    with io.open(os.path.join(HERE, 'README.rst'), 'r') as readme:
        setup(
            name                 = __project__,
            version              = __version__,
            description          = __doc__,
            long_description     = readme.read(),
            classifiers          = __classifiers__,
            author               = __author__,
            author_email         = __author_email__,
            url                  = __url__,
            license              = 'BSD-3-Clause',
            keywords             = __keywords__,
            packages             = find_packages(),
            include_package_data = True,
            platforms            = __platforms__,
            install_requires     = __requires__,
            extras_require       = __extra_requires__,
            entry_points         = __entry_points__,
        )


if __name__ == '__main__':
    main()



# GPIO Zero: a library for controlling the Raspberry Pi's GPIO pins
#
# Copyright (c) 2021 Dave Jones <dave@waveform.org.uk>
#
# SPDX-License-Identifier: BSD-3-Clause

from __future__ import (
    unicode_literals,
    absolute_import,
    print_function,
    division,
)
str = type('')

import io
from collections import Counter
try:
    from itertools import izip as zip, izip_longest as zip_longest
except ImportError:
    from itertools import zip_longest


def load_segment_font(filename_or_obj, width, height, pins):
    """
    A generic function for parsing segment font definition files.

    If you're working with "standard" `7-segment`_ or `14-segment`_ displays
    you *don't* want this function; see :func:`load_font_7seg` or
    :func:`load_font_14seg` instead. However, if you are working with another
    style of segmented display and wish to construct a parser for a custom
    format, this is the function you want.

    The *filename_or_obj* parameter is simply the file-like object or filename
    to load. This is typically passed in from the calling function.

    The *width* and *height* parameters give the width and height in characters
    of each character definition. For example, these are 3 and 3 for 7-segment
    displays. Finally, *pins* is a list of tuples that defines the position of
    each pin definition in the character array, and the character that marks
    that position "active".

    For example, for 7-segment displays this function is called as follows::

        load_segment_font(filename_or_obj, width=3, height=3, pins=[
            (1, '_'), (5, '|'), (8, '|'), (7, '_'),
            (6, '|'), (3, '|'), (4, '_')])

    This dictates that each character will be defined by a 3x3 character grid
    which will be converted into a nine-character string like so:

    .. code-block:: text

        012
        345  ==>  '012345678'
        678

    Position 0 is always assumed to be the character being defined. The *pins*
    list then specifies: the first pin is the character at position 1 which
    will be "on" when that character is "_". The second pin is the character
    at position 5 which will be "on" when that character is "|", and so on.

    .. _7-segment: https://en.wikipedia.org/wiki/Seven-segment_display
    .. _14-segment: https://en.wikipedia.org/wiki/Fourteen-segment_display
    """
    assert 0 < len(pins) <= (width * height) - 1
    if isinstance(filename_or_obj, bytes):
        filename_or_obj = filename_or_obj.decode('utf-8')
    opened = isinstance(filename_or_obj, str)
    if opened:
        filename_or_obj = io.open(filename_or_obj, 'r')
    try:
        lines = filename_or_obj.read()
        if isinstance(lines, bytes):
            lines = lines.decode('utf-8')
        lines = lines.splitlines()
    finally:
        if opened:
            filename_or_obj.close()

    # Strip out comments and blank lines, but remember the original line
    # numbers of each row for error reporting purposes
    rows = [
        (index, line) for index, line in enumerate(lines, start=1)
        # Strip comments and blank (or whitespace) lines
        if line.strip() and not line.startswith('#')
    ]
    line_numbers = {
        row_index: line_index
        for row_index, (line_index, row) in enumerate(rows)
    }
    rows = [row for index, row in rows]
    if len(rows) % height:
        raise ValueError('number of definition lines is not divisible by '
                         '{height}'.format(height=height))

    # Strip out blank columns then transpose back to rows, and make sure
    # everything is the right "shape"
    for n in range(0, len(rows), height):
        cols = [
            col for col in zip_longest(*rows[n:n + height], fillvalue=' ')
            # Strip blank (or whitespace) columns
            if ''.join(col).strip()
        ]
        rows[n:n + height] = list(zip(*cols))
    for row_index, row in enumerate(rows):
        if len(row) % width:
            raise ValueError(
                'length of definitions starting on line {line} is not '
                'divisible by {width}'.format(
                    line=line_numbers[row_index], width=width))

    # Split rows up into character definitions. After this, chars will be a
    # list of strings each with width x height characters. The first character
    # in each string will be the character being defined
    chars = [
        ''.join(
            char
            for row in rows[y::height]
            for char in row
        )[x::width]
        for y in range(height)
        for x in range(width)
    ]
    chars = [''.join(char) for char in zip(*chars)]

    # Strip out blank entries (a consequence of zip_longest above) and check
    # there're no repeat definitions
    chars = [char for char in chars if char.strip()]
    counts = Counter(char[0] for char in chars)
    for char, count in counts.most_common():
        if count > 1:
            raise ValueError(
                'multiple definitions for {char!r}'.format(char=char))

    return {
        char[0]: tuple(int(char[pos] == on) for pos, on in pins)
        for char in chars
    }


def load_font_7seg(filename_or_obj):
    """
    Given a filename or a file-like object, parse it as an font definition for
    a `7-segment display`_, returning a :class:`dict` suitable for use with
    :class:`~gpiozero.LEDCharDisplay`.

    The file-format is a simple text-based format in which blank and #-prefixed
    lines are ignored. All other lines are assumed to be groups of character
    definitions which are cells of 3x3 characters laid out as follows:

    .. code-block:: text

        Ca
        fgb
        edc

    Where C is the character being defined, and a-g define the states of the
    LEDs for that position. a, d, and g are on if they are "_". b, c, e, and
    f are on if they are "|". Any other character in these positions is
    considered off. For example, you might define the following characters:

    .. code-block:: text

         .  0_  1.  2_  3_  4.  5_  6_  7_  8_  9_
        ... |.| ..| ._| ._| |_| |_. |_. ..| |_| |_|
        ... |_| ..| |_. ._| ..| ._| |_| ..| |_| ._|

    In the example above, empty locations are marked with "." but could mostly
    be left as spaces. However, the first item defines the space (" ")
    character and needs *some* non-space characters in its definition as the
    parser also strips empty columns (as typically occur between character
    definitions). This is also why the definition for "1" must include
    something to fill the middle column.

    .. _7-segment display: https://en.wikipedia.org/wiki/Seven-segment_display
    """
    return load_segment_font(filename_or_obj, width=3, height=3, pins=[
        (1, '_'), (5, '|'), (8, '|'), (7, '_'),
        (6, '|'), (3, '|'), (4, '_')])


def load_font_14seg(filename_or_obj):
    """
    Given a filename or a file-like object, parse it as a font definition for a
    `14-segment display`_, returning a :class:`dict` suitable for use with
    :class:`~gpiozero.LEDCharDisplay`.

    The file-format is a simple text-based format in which blank and #-prefixed
    lines are ignored. All other lines are assumed to be groups of character
    definitions which are cells of 5x5 characters laid out as follows:

    .. code-block:: text

        X.a..
        fijkb
        .g.h.
        elmnc
        ..d..

    Where X is the character being defined, and a-n define the states of the
    LEDs for that position. a, d, g, and h are on if they are "-". b, c, e, f,
    j, and m are on if they are "|". i and n are on if they are "\\". Finally,
    k and l are on if they are "/". Any other character in these positions is
    considered off. For example, you might define the following characters:

    .. code-block:: text

         .... 0---  1..   2---  3---  4     5---  6---  7---. 8---  9---
        ..... |  /|    /|     |     | |   | |     |        /  |   | |   |
        ..... | / |     |  ---    --   ---|  ---  |---    |    ---   ---|
        ..... |/  |     | |         |     |     | |   |   |   |   |     |
        .....  ---         ---   ---         ---   ---         ---

    In the example above, several locations have extraneous characters. For
    example, the "/" in the center of the "0" definition, or the "-" in the
    middle of the "8". These locations are ignored, but filled in nonetheless
    to make the shape more obvious.

    These extraneous locations could equally well be left as spaces. However,
    the first item defines the space (" ") character and needs *some* non-space
    characters in its definition as the parser also strips empty columns (as
    typically occur between character definitions) and verifies that
    definitions are 5 columns wide and 5 rows high.

    This also explains why place-holder characters (".") have been inserted at
    the top of the definition of the "1" character. Otherwise the parser will
    strip these empty columns and decide the definition is invalid (as the
    result is only 3 columns wide).

    .. _14-segment display: https://en.wikipedia.org/wiki/Fourteen-segment_display
    """
    return load_segment_font(filename_or_obj, width=5, height=5, pins=[
        (2, '-'),  (9, '|'),  (19, '|'), (22, '-'),
        (15, '|'), (5, '|'),  (11, '-'), (13, '-'),
        (6, '\\'), (7, '|'),  (8, '/'),  (16, '/'),
        (17, '|'), (18, '\\')])



# vim: set fileencoding=utf-8:
#
# GPIO Zero: a library for controlling the Raspberry Pi's GPIO pins
#
# Copyright (c) 2016-2021 Dave Jones <dave@waveform.org.uk>
# Copyright (c) 2016 Andrew Scheller <github@loowis.durge.org>
#
# SPDX-License-Identifier: BSD-3-Clause

from __future__ import (
    unicode_literals,
    print_function,
    absolute_import,
    division,
    )
str = type('')

from threading import Thread, Event

from .exc import ZombieThread


_THREADS = set()
def _threads_shutdown():
    while _THREADS:
        threads = _THREADS.copy()
        # Optimization: instead of calling stop() which implicitly calls
        # join(), set all the stopping events simultaneously, *then* join
        # threads with a reasonable timeout
        for t in threads:
            t.stopping.set()
        for t in threads:
            t.join(10)


class GPIOThread(Thread):
    def __init__(self, target, args=(), kwargs=None, name=None):
        if kwargs is None:
            kwargs = {}
        self.stopping = Event()
        super(GPIOThread, self).__init__(None, target, name, args, kwargs)
        self.daemon = True

    def start(self):
        self.stopping.clear()
        _THREADS.add(self)
        super(GPIOThread, self).start()

    def stop(self, timeout=10):
        self.stopping.set()
        self.join(timeout)

    def join(self, timeout=None):
        super(GPIOThread, self).join(timeout)
        if self.is_alive():
            assert timeout is not None
            # timeout can't be None here because if it was, then join()
            # wouldn't return until the thread was dead
            raise ZombieThread(
                "Thread failed to die within %d seconds" % timeout)
        else:
            _THREADS.discard(self)



# vim: set fileencoding=utf-8:
#
# GPIO Zero: a library for controlling the Raspberry Pi's GPIO pins
#
# Copyright (c) 2016-2021 Dave Jones <dave@waveform.org.uk>
# Copyright (c) 2016 Andrew Scheller <github@loowis.durge.org>
#
# SPDX-License-Identifier: BSD-3-Clause

from __future__ import (
    unicode_literals,
    absolute_import,
    print_function,
    division,
    )
str = type('')


import gc
import sys
import pytest
import random
import weakref

from gpiozero.compat import *


def test_frozendict():
    d = {1: 'a', 2: 'b'}
    e = {1: 'a', 2: 'b', 'foo': 'bar'}
    f = frozendict(d)
    assert f[1] == 'a'
    assert f[2] == 'b'
    with pytest.raises(KeyError):
        f[3]
    with pytest.raises(TypeError):
        f[3] = 'c'
    assert d == f
    assert d == f.copy()
    assert e == f.copy(foo='bar')
    assert len(f) == 2
    assert {k: v for k, v in f.items()} == d
    h = hash(f)
    assert h is not None
    assert hash(f) == h
    assert repr(f) == "<frozendict %s>" % repr(d)

# ported from the official test cases; see
# https://github.com/python/cpython/blob/master/Lib/test/test_math.py for original

NAN = float('nan')
INF = float('inf')
NINF = float('-inf')

def test_isclose_negative_tolerances():
    with pytest.raises(ValueError):
        isclose(1, 1, rel_tol=-1e-100)
    with pytest.raises(ValueError):
        isclose(1, 1, rel_tol=1e-100, abs_tol=-1e10)

def test_isclose_identical():
    examples = [
        (2.0, 2.0),
        (0.1e200, 0.1e200),
        (1.123e-300, 1.123e-300),
        (12345, 12345.0),
        (0.0, -0.0),
        (345678, 345678),
        ]
    for a, b in examples:
        assert isclose(a, b, rel_tol=0.0, abs_tol=0.0)

def test_isclose_eight_decimals():
    examples = [
        (1e8, 1e8 + 1),
        (-1e-8, -1.000000009e-8),
        (1.12345678, 1.12345679),
        ]
    for a, b in examples:
        assert isclose(a, b, rel_tol=1e-8)
        assert not isclose(a, b, rel_tol=1e-9)

def test_isclose_near_zero():
    examples = [1e-9, -1e-9, -1e-150]
    for a in examples:
        assert not isclose(a, 0.0, rel_tol=0.9)
        assert isclose(a, 0.0, abs_tol=1e-8)

def test_isclose_inf():
    assert isclose(INF, INF)
    assert isclose(INF, INF, abs_tol=0.0)
    assert isclose(NINF, NINF)
    assert isclose(NINF, NINF, abs_tol=0.0)

def test_isclose_inf_ninf_nan():
    examples = [
        (NAN, NAN),
        (NAN, 1e-100),
        (1e-100, NAN),
        (INF, NAN),
        (NAN, INF),
        (INF, NINF),
        (INF, 1.0),
        (1.0, INF),
        (INF, 1e308),
        (1e308, INF),
        ]
    for a, b in examples:
        assert not isclose(a, b, abs_tol=0.999999999999999)

def test_isclose_zero_tolerance():
    examples = [
        (1.0, 1.0),
        (-3.4, -3.4),
        (-1e-300, -1e-300),
        ]
    for a, b in examples:
        assert isclose(a, b, rel_tol=0.0)
    examples = [
        (1.0, 1.000000000000001),
        (0.99999999999999, 1.0),
        (1.0e200, .999999999999999e200),
        ]
    for a, b in examples:
        assert not isclose(a, b, rel_tol=0.0)

def test_isclose_assymetry():
    assert isclose(9, 10, rel_tol=0.1)
    assert isclose(10, 9, rel_tol=0.1)

def test_isclose_integers():
    examples = [
        (100000001, 100000000),
        (123456789, 123456788),
        ]
    for a, b in examples:
        assert isclose(a, b, rel_tol=1e-8)
        assert not isclose(a, b, rel_tol=1e-9)

# ported from the official test cases; see
# https://github.com/python/cpython/blob/master/Lib/test/test_statistics.py for
# original

def test_mean():
    examples = [
        (4.8125, (0, 1, 2, 3, 3, 3, 4, 5, 5, 6, 7, 7, 7, 7, 8, 9)),
        (22.015625, (17.25, 19.75, 20.0, 21.5, 21.75, 23.25, 25.125, 27.5)),
        (INF, (1, 3, 5, 7, 9, INF)),
        (NINF, (1, 3, 5, 7, 9, NINF)),
        ]
    for result, values in examples:
        values = list(values)
        random.shuffle(values)
        assert mean(values) == result
        assert mean(iter(values)) == result

def test_mean_big_data():
    c = 1e9
    data = [3.4, 4.5, 4.9, 6.7, 6.8, 7.2, 8.0, 8.1, 9.4]
    expected = mean(data) + c
    assert expected != c
    assert mean([x + c for x in data]) == expected

def test_mean_doubled_data():
    data = [random.uniform(-3, 5) for _ in range(1000)]
    expected = mean(data)
    actual = mean(data * 2)
    assert isclose(expected, actual)

def test_mean_empty():
    with pytest.raises(ValueError):
        mean(())

def test_median():
    assert median([1, 2, 3, 4, 5, 6]) == 3.5
    assert median([1, 2, 3, 4, 5, 6, 9]) == 4

def test_median_empty():
    with pytest.raises(ValueError):
        median(())

# ported from the official test cases; see
# https://github.com/python/cpython/blob/master/Lib/test/test_weakref.py for
# original

class Object(object):
    def __init__(self, arg):
        self.arg = arg
    def __repr__(self):
        return "<Object %r>" % self.arg
    def __eq__(self, other):
        if isinstance(other, Object):
            return self.arg == other.arg
        return NotImplemented
    def __ne__(self, other):
        if isinstance(other, Object):
            return self.arg != other.arg
        return NotImplemented
    def __lt__(self, other):
        if isinstance(other, Object):
            return self.arg < other.arg
        return NotImplemented
    def __hash__(self):
        return hash(self.arg)
    def some_method(self):
        return 4
    def other_method(self):
        return 5

@pytest.fixture()
def subclass(request):
    class C(Object):
        def some_method(self):
            return 6
    return C


def test_weakmethod_alive():
    o = Object(1)
    r = WeakMethod(o.some_method)
    assert isinstance(r, weakref.ReferenceType)
    assert isinstance(r(), type(o.some_method))
    assert r().__self__ is o
    assert r().__func__ is o.some_method.__func__
    assert r()() == 4

def test_weakmethod_object_dead():
    o = Object(1)
    r = WeakMethod(o.some_method)
    del o
    gc.collect()
    assert r() is None

@pytest.mark.xfail((3, 2) <= sys.version_info[:2] <= (3, 3),
                   reason='intermittent failure on py3.2 and py3.3')
def test_weakmethod_method_dead(subclass):
    o = subclass(1)
    r = WeakMethod(o.some_method)
    del subclass.some_method
    gc.collect()
    assert r() is None

def test_weakmethod_callback_when_object_dead(subclass):
    calls = []
    def cb(arg):
        calls.append(arg)
    o = subclass(1)
    r = WeakMethod(o.some_method, cb)
    del o
    gc.collect()
    assert calls == [r]
    # Callback is only called once
    subclass.some_method = Object.some_method
    gc.collect()
    assert calls == [r]

@pytest.mark.xfail((3, 2) <= sys.version_info[:2] <= (3, 3),
                   reason='intermittent failure on py3.2 and py3.3')
def test_weakmethod_callback_when_method_dead(subclass):
    calls = []
    def cb(arg):
        calls.append(arg)
    o = subclass(1)
    r = WeakMethod(o.some_method, cb)
    del subclass.some_method
    gc.collect()
    assert calls == [r]
    # Callback is only called once
    del o
    gc.collect()
    assert calls == [r]

@pytest.mark.xfail(hasattr(sys, 'pypy_version_info'),
                   reason='pypy memory management is different')
def test_weakmethod_no_cycles():
    o = Object(1)
    def cb(_):
        pass
    r = WeakMethod(o.some_method, cb)
    wr = weakref.ref(r)
    del r
    assert wr() is None

def test_weakmethod_equality():
    def _eq(a, b):
        assert a == b
        assert not (a != b)
    def _ne(a, b):
        assert not (a == b)
        assert a != b
    x = Object(1)
    y = Object(1)
    a = WeakMethod(x.some_method)
    b = WeakMethod(y.some_method)
    c = WeakMethod(x.other_method)
    d = WeakMethod(y.other_method)
    # Objects equal, same method
    _eq(a, b)
    _eq(c, d)
    # Objects equal, different method
    _ne(a, c)
    _ne(a, d)
    _ne(b, c)
    _ne(b, d)
    # Objects unequal, same or different method
    z = Object(2)
    e = WeakMethod(z.some_method)
    f = WeakMethod(z.other_method)
    _ne(a, e)
    _ne(a, f)
    _ne(b, e)
    _ne(b, f)
    del x, y, z
    gc.collect()
    # Dead WeakMethods compare by identity
    refs = a, b, c, d, e, f
    for q in refs:
        for r in refs:
            assert (q == r) == (q is r)
            assert (q != r) == (q is not r)

def test_weakmethod_hashing():
    x = Object(1)
    y = Object(1)
    a = WeakMethod(x.some_method)
    b = WeakMethod(y.some_method)
    c = WeakMethod(y.other_method)
    # Since WeakMethod objects are equal, the hashes should be equal
    assert hash(a) == hash(b)
    ha = hash(a)
    # Dead WeakMethods retain their old hash value
    del x, y
    gc.collect()
    assert hash(a) == ha
    assert hash(b) == ha
    # If it wasn't hashed when alive, a dead WeakMethod cannot be hashed
    with pytest.raises(TypeError):
        hash(c)

def test_weakmethod_bad_method():
    with pytest.raises(TypeError):
        WeakMethod('foo')

def test_weakmethod_other_equality():
    x = Object(1)
    a = WeakMethod(x.some_method)
    b = WeakMethod(x.other_method)
    assert not a == 1
    assert a != 1



from gpiozero import Robot
from evdev import InputDevice, list_devices, ecodes

robot = Robot(left=(4, 14), right=(17, 18))

# Get the list of available input devices
devices = [InputDevice(device) for device in list_devices()]
# Filter out everything that's not a keyboard. Keyboards are defined as any
# device which has keys, and which specifically has keys 1..31 (roughly Esc,
# the numeric keys, the first row of QWERTY plus a few more) and which does
# *not* have key 0 (reserved)
must_have = {i for i in range(1, 32)}
must_not_have = {0}
devices = [
    dev
    for dev in devices
    for keys in (set(dev.capabilities().get(ecodes.EV_KEY, [])),)
    if must_have.issubset(keys)
    and must_not_have.isdisjoint(keys)
]
# Pick the first keyboard
keyboard = devices[0]

keypress_actions = {
    ecodes.KEY_UP: robot.forward,
    ecodes.KEY_DOWN: robot.backward,
    ecodes.KEY_LEFT: robot.left,
    ecodes.KEY_RIGHT: robot.right,
}

for event in keyboard.read_loop():
    if event.type == ecodes.EV_KEY and event.code in keypress_actions:
        if event.value == 1:  # key pressed
            keypress_actions[event.code]()
        if event.value == 0:  # key released
            robot.stop()



from gpiozero import MCP3008
from time import sleep

def convert_temp(gen):
    for value in gen:
        yield (value * 3.3 - 0.5) * 100

adc = MCP3008(channel=0)

for temp in convert_temp(adc.values):
    print('The temperature is', temp, 'C')
    sleep(1)



from gpiozero import Button
import pygame.mixer
from pygame.mixer import Sound
from signal import pause

pygame.mixer.init()

button_sounds = {
    Button(2): Sound("samples/drum_tom_mid_hard.wav"),
    Button(3): Sound("samples/drum_cymbal_open.wav"),
}

for button, sound in button_sounds.items():
    button.when_pressed = sound.play

pause()



# vim: set fileencoding=utf-8:
#
# GPIO Zero: a library for controlling the Raspberry Pi's GPIO pins
#
# Copyright (c) 2019-2021 Dave Jones <dave@waveform.org.uk>
#
# SPDX-License-Identifier: BSD-3-Clause

from __future__ import (
    unicode_literals,
    absolute_import,
    print_function,
    division,
)
str = type('')

import warnings
from fractions import Fraction

import pytest

from gpiozero.exc import AmbiguousTone
from gpiozero.tones import Tone


@pytest.fixture
def A4(request):
    return Tone.from_frequency(440.0)

def test_tone_init(A4):
    with warnings.catch_warnings(record=True) as w:
        warnings.resetwarnings()
        assert Tone(440) == A4
        assert Tone("A4") == A4
        assert Tone(Fraction(880, 2)) == A4
        assert len(w) == 0
        assert Tone(69) == A4
        assert len(w) == 1
        assert Tone(0) != A4
        assert len(w) == 1
        assert isinstance(w[0].message, AmbiguousTone)
    assert Tone(frequency=440) == A4
    assert Tone(note="A4") == A4
    assert Tone(midi=69) == A4
    with pytest.raises(TypeError):
        Tone()
    with pytest.raises(TypeError):
        Tone(foo=1)
    with pytest.raises(TypeError):
        Tone(frequency=440, midi=69)
    with pytest.raises(TypeError):
        Tone(440, midi=69)

def test_tone_str(A4):
    assert str(A4) == "A4"
    assert str(A4.up()) == "A#4"
    assert str(A4.down(12)) == "A3"
    assert repr(A4) == "<Tone %s midi=69 frequency=440.00Hz>" % ('note=%r' % 'A4')
    assert repr(Tone(13000)) == '<Tone frequency=13000.00Hz>'

def test_tone_from_frequency(A4):
    assert Tone.from_frequency(440) == A4
    assert Tone.from_frequency(440.0) == A4
    with pytest.raises(ValueError):
        Tone.from_frequency(-100.0)
    with pytest.raises(ValueError):
        Tone.from_frequency(30000.0)

def test_tone_from_note(A4):
    assert Tone.from_note(b"A4") == A4
    assert Tone.from_note("A4") == A4
    with pytest.raises(ValueError):
        Tone.from_note("a4")
    with pytest.raises(ValueError):
        Tone.from_note("foo")
    with pytest.raises(ValueError):
        Tone.from_note(0)

def test_tone_from_midi(A4):
    assert Tone.from_midi(69) == A4
    with pytest.raises(ValueError):
        Tone.from_midi(500)
    with pytest.raises(ValueError):
        Tone.from_midi(-1)

def test_tone_frequency(A4):
    assert A4.frequency == 440.0
    assert A4.up(12).frequency == 880.0
    assert A4.down(12).frequency == 220.0

def test_tone_midi(A4):
    assert A4.midi == 69
    assert A4.up().midi == 70
    assert A4.down().midi == 68
    with pytest.raises(ValueError):
        Tone.from_frequency(2).midi
    with pytest.raises(ValueError):
        Tone.from_frequency(15000).midi

def test_tone_note(A4):
    assert A4.note == "A4"
    assert A4.up().note == "A#4"
    assert A4.down().note == "G#4"
    with pytest.raises(ValueError):
        Tone.from_midi(8).note



# vim: set fileencoding=utf-8:
#
# GPIO Zero: a library for controlling the Raspberry Pi's GPIO pins
#
# Copyright (c) 2016-2021 Dave Jones <dave@waveform.org.uk>
# Copyright (c) 2019 Andrew Scheller <github@loowis.durge.org>
#
# SPDX-License-Identifier: BSD-3-Clause

from __future__ import (
    unicode_literals,
    absolute_import,
    print_function,
    division,
    )
nstr = str
str = type('')


import io
import sys
import pytest
from array import array
from mock import patch
from collections import namedtuple

from gpiozero.pins.native import NativeFactory
from gpiozero.pins.local import (
    LocalPiHardwareSPI,
    LocalPiSoftwareSPI,
    LocalPiHardwareSPIShared,
    LocalPiSoftwareSPIShared,
    )
from gpiozero.pins.mock import MockSPIDevice
from gpiozero import *


def test_spi_hardware_params(mock_factory):
    with patch('gpiozero.pins.local.SpiDev'):
        with mock_factory.spi() as device:
            assert isinstance(device, LocalPiHardwareSPI)
            assert repr(device) == 'SPI(port=0, device=0)'
            device.close()
            assert device.closed
            assert repr(device) == 'SPI(closed)'
        with mock_factory.spi(port=0, device=0) as device:
            assert isinstance(device, LocalPiHardwareSPI)
        with mock_factory.spi(port=0, device=1) as device:
            assert isinstance(device, LocalPiHardwareSPI)
        with mock_factory.spi(clock_pin=11) as device:
            assert isinstance(device, LocalPiHardwareSPI)
        with mock_factory.spi(clock_pin=11, mosi_pin=10, select_pin=8) as device:
            assert isinstance(device, LocalPiHardwareSPI)
        with mock_factory.spi(clock_pin=11, mosi_pin=10, select_pin=7) as device:
            assert isinstance(device, LocalPiHardwareSPI)
        # Ensure we support "partial" SPI where we don't reserve a pin because
        # the device wants it for general IO (see SPI screens which use a pin
        # for data/commands)
        with mock_factory.spi(clock_pin=11, mosi_pin=10, miso_pin=None, select_pin=7) as device:
            assert isinstance(device, LocalPiHardwareSPI)
        with mock_factory.spi(clock_pin=11, mosi_pin=None, miso_pin=9, select_pin=7) as device:
            assert isinstance(device, LocalPiHardwareSPI)
        with mock_factory.spi(shared=True) as device:
            assert isinstance(device, LocalPiHardwareSPIShared)
        with pytest.raises(ValueError):
            mock_factory.spi(port=1)
        with pytest.raises(ValueError):
            mock_factory.spi(device=2)
        with pytest.raises(ValueError):
            mock_factory.spi(port=0, clock_pin=12)
        with pytest.raises(ValueError):
            mock_factory.spi(foo='bar')

def test_spi_software_params(mock_factory):
    with patch('gpiozero.pins.local.SpiDev'):
        with mock_factory.spi(select_pin=6) as device:
            assert isinstance(device, LocalPiSoftwareSPI)
            assert repr(device) == 'SPI(clock_pin=11, mosi_pin=10, miso_pin=9, select_pin=6)'
            device.close()
            assert device.closed
            assert repr(device) == 'SPI(closed)'
        with mock_factory.spi(clock_pin=11, mosi_pin=9, miso_pin=10) as device:
            assert isinstance(device, LocalPiSoftwareSPI)
            device._bus.close()
            assert device._bus.closed
            device.close()
            assert device.closed
        with mock_factory.spi(select_pin=6, shared=True) as device:
            assert isinstance(device, LocalPiSoftwareSPIShared)
    with patch('gpiozero.pins.local.SpiDev', None):
        # Clear out the old factory's caches (this is only necessary
        # because we're being naughty switching out patches)
        mock_factory.pins.clear()
        mock_factory._reservations.clear()
        # Ensure software fallback works when SpiDev isn't present
        with mock_factory.spi() as device:
            assert isinstance(device, LocalPiSoftwareSPI)

def test_spi_hardware_conflict(mock_factory):
    with patch('gpiozero.pins.local.SpiDev') as spidev:
        with LED(11) as led:
            with pytest.raises(GPIOPinInUse):
                mock_factory.spi(port=0, device=0)
    with patch('gpiozero.pins.local.SpiDev') as spidev:
        with mock_factory.spi(port=0, device=0) as spi:
            with pytest.raises(GPIOPinInUse):
                LED(11)

def test_spi_software_same_bus(mock_factory):
    with patch('gpiozero.pins.local.SpiDev'):
        with mock_factory.spi(select_pin=6) as device:
            with pytest.raises(GPIOPinInUse):
                mock_factory.spi(select_pin=6)
            with mock_factory.spi(select_pin=5) as another_device:
                assert device._bus is another_device._bus

def test_spi_software_shared_bus(mock_factory):
    with patch('gpiozero.pins.local.SpiDev'):
        with mock_factory.spi(select_pin=6, shared=True) as device:
            with mock_factory.spi(select_pin=6, shared=True) as another_device:
                assert device is another_device

def test_spi_hardware_read(mock_factory):
    with patch('gpiozero.pins.local.SpiDev') as spidev:
        spidev.return_value.xfer2.side_effect = lambda data: list(range(10))[:len(data)]
        with mock_factory.spi() as device:
            assert device.read(3) == [0, 1, 2]
            assert device.read(6) == list(range(6))

def test_spi_hardware_write(mock_factory):
    with patch('gpiozero.pins.local.SpiDev') as spidev:
        spidev.return_value.xfer2.side_effect = lambda data: list(range(10))[:len(data)]
        with mock_factory.spi() as device:
            assert device.write([0, 1, 2]) == 3
            assert spidev.return_value.xfer2.called_with([0, 1, 2])
            assert device.write(list(range(6))) == 6
            assert spidev.return_value.xfer2.called_with(list(range(6)))

def test_spi_hardware_modes(mock_factory):
    with patch('gpiozero.pins.local.SpiDev') as spidev:
        spidev.return_value.mode = 0
        spidev.return_value.lsbfirst = False
        spidev.return_value.cshigh = True
        spidev.return_value.bits_per_word = 8
        with mock_factory.spi() as device:
            assert device.clock_mode == 0
            assert not device.clock_polarity
            assert not device.clock_phase
            device.clock_polarity = False
            assert device.clock_mode == 0
            device.clock_polarity = True
            assert device.clock_mode == 2
            device.clock_phase = True
            assert device.clock_mode == 3
            assert not device.lsb_first
            assert device.select_high
            assert device.bits_per_word == 8
            device.select_high = False
            device.lsb_first = True
            device.bits_per_word = 12
            assert not spidev.return_value.cshigh
            assert spidev.return_value.lsbfirst
            assert spidev.return_value.bits_per_word == 12
            device.rate = 1000000
            assert device.rate == 1000000
            device.rate = 500000
            assert device.rate == 500000

def test_spi_software_read(mock_factory):
    class SPISlave(MockSPIDevice):
        def on_start(self):
            super(SPISlave, self).on_start()
            for i in range(10):
                self.tx_word(i)
    with patch('gpiozero.pins.local.SpiDev', None), \
            SPISlave(11, 10, 9, 8) as slave, \
            mock_factory.spi() as master:
        assert master.read(3) == [0, 1, 2]
        assert master.read(6) == [0, 1, 2, 3, 4, 5]
        slave.clock_phase = True
        master.clock_phase = True
        assert master.read(3) == [0, 1, 2]
        assert master.read(6) == [0, 1, 2, 3, 4, 5]

def test_spi_software_write(mock_factory):
    with patch('gpiozero.pins.local.SpiDev', None), \
            MockSPIDevice(11, 10, 9, 8) as test_device, \
            mock_factory.spi() as master:
        master.write([0])
        assert test_device.rx_word() == 0
        master.write([2, 0])
        # 0b 0000_0010 0000_0000
        assert test_device.rx_word() == 512
        master.write([0, 1, 1])
        # 0b 0000_0000 0000_0001 0000_0001
        assert test_device.rx_word() == 257

def test_spi_software_write_lsb_first(mock_factory):
    with patch('gpiozero.pins.local.SpiDev', None), \
            MockSPIDevice(11, 10, 9, 8, lsb_first=True) as test_device, \
            mock_factory.spi() as master:
        # lsb_first means the bit-strings above get reversed
        master.write([0])
        assert test_device.rx_word() == 0
        master.write([2, 0])
        # 0b 0000_0000 0100_0000
        assert test_device.rx_word() == 64
        master.write([0, 1, 1])
        # 0b 1000_0000 1000_0000 0000_0000
        assert test_device.rx_word() == 8421376

def test_spi_software_clock_mode(mock_factory):
    with patch('gpiozero.pins.local.SpiDev', None), \
            mock_factory.spi() as master:
        assert master.clock_mode == 0
        assert not master.clock_polarity
        assert not master.clock_phase
        master.clock_polarity = False
        assert master.clock_mode == 0
        master.clock_polarity = True
        assert master.clock_mode == 2
        master.clock_phase = True
        assert master.clock_mode == 3
        master.clock_mode = 0
        assert not master.clock_polarity
        assert not master.clock_phase
        with pytest.raises(ValueError):
            master.clock_mode = 5

def test_spi_software_attr(mock_factory):
    with patch('gpiozero.pins.local.SpiDev', None), \
            mock_factory.spi() as master:
        assert not master.lsb_first
        assert not master.select_high
        assert master.bits_per_word == 8
        master.bits_per_word = 12
        assert master.bits_per_word == 12
        master.lsb_first = True
        assert master.lsb_first
        master.select_high = True
        assert master.select_high
        with pytest.raises(ValueError):
            master.bits_per_word = 0


# XXX Test two simultaneous SPI devices sharing clock, MOSI, and MISO, with
# separate select pins (including threaded tests which attempt simultaneous
# reading/writing)



from itertools import cycle
from collections import deque
from gpiozero import LEDMultiCharDisplay
from signal import pause

display = LEDMultiCharDisplay(
    LEDCharDisplay(22, 23, 24, 25, 21, 20, 16, dp=12), 26, 19, 13, 6)

def scroller(message, chars=4):
    d = deque(maxlen=chars)
    for c in cycle(message):
        d.append(c)
        if len(d) == chars:
            yield ''.join(d)

display.source_delay = 0.2
display.source = scroller('GPIO 2ER0    ')
pause()



from gpiozero import Button
from signal import pause

def say_hello():
    print("Hello!")

def say_goodbye():
    print("Goodbye!")

button = Button(2)

button.when_pressed = say_hello
button.when_released = say_goodbye

pause()



# GPIO Zero: a library for controlling the Raspberry Pi's GPIO pins
#
# Copyright (c) 2021 Dave Jones <dave@waveform.org.uk>
#
# SPDX-License-Identifier: BSD-3-Clause

from __future__ import (
    unicode_literals,
    absolute_import,
    print_function,
    division,
    )
str = type('')

import os

import lgpio

from . import SPI
from .pi import spi_port_device
from .local import LocalPiFactory, LocalPiPin
from ..mixins import SharedMixin
from ..exc import (
    PinInvalidFunction,
    PinSetInput,
    PinFixedPull,
    PinInvalidPull,
    PinInvalidBounce,
    PinInvalidState,
    SPIBadArgs,
    SPIInvalidClockMode,
    PinPWMFixedValue,
    DeviceClosed
)


class LGPIOFactory(LocalPiFactory):
    """
    Extends :class:`~gpiozero.pins.local.LocalPiFactory`. Uses the `lgpio`_
    library to interface to the local computer's GPIO pins. The lgpio library
    simply talks to Linux gpiochip devices; it is not specific to the Raspberry
    Pi although this class is currently constructed under the assumption that
    it is running on a Raspberry Pi.

    You can construct lgpio pins manually like so::

        from gpiozero.pins.lgpio import LGPIOFactory
        from gpiozero import LED

        factory = LGPIOFactory(chip=0)
        led = LED(12, pin_factory=factory)

    The *chip* parameter to the factory constructor specifies which gpiochip
    device to attempt to open. It defaults to 0 and thus doesn't normally need
    to be specified (the example above only includes it for completeness).

    The lgpio library relies on access to the :file:`/dev/gpiochip*` devices.
    If you run into issues, please check that your user has read/write access
    to the specific gpiochip device you are attempting to open (0 by default).

    .. _lgpio: http://abyz.me.uk/lg/py_lgpio.html
    """
    def __init__(self, chip=0):
        super(LGPIOFactory, self).__init__()
        self._handle = lgpio.gpiochip_open(chip)
        self._chip = chip
        self.pin_class = LGPIOPin

    def close(self):
        super(LGPIOFactory, self).close()
        if self._handle is not None:
            lgpio.gpiochip_close(self._handle)
            self._handle = None

    @property
    def chip(self):
        return self._chip

    def _get_spi_class(self, shared, hardware):
        # support via lgpio instead of spidev
        if hardware:
            return [LGPIOHardwareSPI, LGPIOHardwareSPIShared][shared]
        return super(LGPIOFactory, self)._get_spi_class(shared, hardware=False)


class LGPIOPin(LocalPiPin):
    """
    Extends :class:`~gpiozero.pins.local.LocalPiPin`. Pin implementation for
    the `lgpio`_ library. See :class:`LGPIOFactory` for more information.

    .. _lgpio: http://abyz.me.uk/lg/py_lgpio.html
    """
    GPIO_IS_KERNEL         = 1 << 0
    GPIO_IS_OUT            = 1 << 1
    GPIO_IS_ACTIVE_LOW     = 1 << 2
    GPIO_IS_OPEN_DRAIN     = 1 << 3
    GPIO_IS_OPEN_SOURCE    = 1 << 4
    GPIO_IS_BIAS_PULL_UP   = 1 << 5
    GPIO_IS_BIAS_PULL_DOWN = 1 << 6
    GPIO_IS_BIAS_DISABLE   = 1 << 7
    GPIO_IS_LG_INPUT       = 1 << 8
    GPIO_IS_LG_OUTPUT      = 1 << 9
    GPIO_IS_LG_ALERT       = 1 << 10
    GPIO_IS_LG_GROUP       = 1 << 11

    GPIO_LINE_FLAGS_MASK = (
        GPIO_IS_ACTIVE_LOW | GPIO_IS_OPEN_DRAIN | GPIO_IS_OPEN_SOURCE |
        GPIO_IS_BIAS_PULL_UP | GPIO_IS_BIAS_PULL_DOWN | GPIO_IS_BIAS_DISABLE)

    GPIO_EDGES = {
        'both':    lgpio.BOTH_EDGES,
        'rising':  lgpio.RISING_EDGE,
        'falling': lgpio.FALLING_EDGE,
        }

    GPIO_EDGES_NAMES = {v: k for (k, v) in GPIO_EDGES.items()}

    def __init__(self, factory, number):
        super(LGPIOPin, self).__init__(factory, number)
        self._pwm = None
        self._bounce = None
        self._callback = None
        self._edges = lgpio.BOTH_EDGES
        lgpio.gpio_claim_input(
            self.factory._handle, self.number, lgpio.SET_BIAS_DISABLE)

    def close(self):
        if self.factory._handle is not None:
            # Closing is really just "resetting" the function of the pin;
            # we let the factory close deal with actually freeing stuff
            lgpio.gpio_claim_input(
                self.factory._handle, self.number, lgpio.SET_BIAS_DISABLE)

    def _get_function(self):
        mode = lgpio.gpio_get_mode(self.factory._handle, self.number)
        return ['input', 'output'][bool(mode & self.GPIO_IS_OUT)]

    def _set_function(self, value):
        if self._callback is not None:
            self._callback.cancel()
            self._callback = None
        try:
            {
                'input': lgpio.gpio_claim_input,
                'output': lgpio.gpio_claim_output,
            }[value](self.factory._handle, self.number)
        except KeyError:
            raise PinInvalidFunction('invalid function "%s" for pin %r' % (value, self))

    def _get_state(self):
        if self._pwm:
            return self._pwm[1] / 100
        else:
            return bool(lgpio.gpio_read(self.factory._handle, self.number))

    def _set_state(self, value):
        if self._pwm:
            freq, duty = self._pwm
            self._pwm = (freq, int(value * 100))
            try:
                lgpio.tx_pwm(self.factory._handle, self.number, *self._pwm)
            except lgpio.error:
                raise PinInvalidState('invalid state "%s" for pin %r' % (value, self))
        elif self.function == 'input':
            raise PinSetInput('cannot set state of pin %r' % self)
        else:
            lgpio.gpio_write(self.factory._handle, self.number, bool(value))

    def _get_pull(self):
        mode = lgpio.gpio_get_mode(self.factory._handle, self.number)
        if mode & self.GPIO_IS_BIAS_PULL_UP:
            return 'up'
        elif mode & self.GPIO_IS_BIAS_PULL_DOWN:
            return 'down'
        else:
            return 'floating'

    def _set_pull(self, value):
        if self.function != 'input':
            raise PinFixedPull('cannot set pull on non-input pin %r' % self)
        if value != 'up' and self.factory.pi_info.pulled_up(repr(self)):
            raise PinFixedPull('%r has a physical pull-up resistor' % self)
        try:
            flags = {
                'up': lgpio.SET_BIAS_PULL_UP,
                'down': lgpio.SET_BIAS_PULL_DOWN,
                'floating': lgpio.SET_BIAS_DISABLE,
            }[value]
        except KeyError:
            raise PinInvalidPull('invalid pull "%s" for pin %r' % (value, self))
        else:
            # Simply calling gpio_claim_input is insufficient to change the
            # line flags on a pin; it needs to be freed and re-claimed
            lgpio.gpio_free(self.factory._handle, self.number)
            lgpio.gpio_claim_input(self.factory._handle, self.number, flags)

    def _get_frequency(self):
        if self._pwm:
            freq, duty = self._pwm
            return freq
        else:
            return None

    def _set_frequency(self, value):
        if not self._pwm and value is not None and value > 0:
            if self.function != 'output':
                raise PinPWMFixedValue('cannot start PWM on pin %r' % self)
            lgpio.tx_pwm(self.factory._handle, self.number, value, 0)
            self._pwm = (value, 0)
        elif self._pwm and value is not None and value > 0:
            freq, duty = self._pwm
            lgpio.tx_pwm(self.factory._handle, self.number, value, duty)
            self._pwm = (value, duty)
        elif self._pwm and (value is None or value == 0):
            lgpio.tx_pwm(self.factory._handle, self.number, 0, 0)
            self._pwm = None

    def _get_bounce(self):
        return None if not self._bounce else self._bounce / 1000000

    def _set_bounce(self, value):
        if value is None:
            value = 0
        elif value < 0:
            raise PinInvalidBounce('bounce must be 0 or greater')
        value = int(value * 1000000)
        lgpio.gpio_set_debounce_micros(self.factory._handle, self.number, value)
        self._bounce = value

    def _get_edges(self):
        return self.GPIO_EDGES_NAMES[self._edges]

    def _set_edges(self, value):
        f = self.when_changed
        self.when_changed = None
        try:
            self._edges = self.GPIO_EDGES[value]
        finally:
            self.when_changed = f

    def _call_when_changed(self, chip, gpio, level, ticks):
        super(LGPIOPin, self)._call_when_changed(ticks / 1000000000, level)

    def _enable_event_detect(self):
        lgpio.gpio_claim_alert(
            self.factory._handle, self.number, self._edges,
            lgpio.gpio_get_mode(self.factory._handle, self.number) &
            self.GPIO_LINE_FLAGS_MASK)
        self._callback = lgpio.callback(
            self.factory._handle, self.number, self._edges,
            self._call_when_changed)

    def _disable_event_detect(self):
        if self._callback is not None:
            self._callback.cancel()
            self._callback = None
        lgpio.gpio_claim_input(
            self.factory._handle, self.number,
            lgpio.gpio_get_mode(self.factory._handle, self.number) &
            self.GPIO_LINE_FLAGS_MASK)


class LGPIOHardwareSPI(SPI):
    """
    Hardware SPI implementation for the `lgpio`_ library. Uses the ``spi_*``
    functions from the lgpio API.

    .. _lgpio: http://abyz.me.uk/lg/py_lgpio.html
    """
    def __init__(self, clock_pin, mosi_pin, miso_pin, select_pin, pin_factory):
        port, device = spi_port_device(
            clock_pin, mosi_pin, miso_pin, select_pin)
        self._port = port
        self._device = device
        self._baud = 500000
        self._spi_flags = 0
        self._handle = None
        super(LGPIOHardwareSPI, self).__init__(pin_factory=pin_factory)
        to_reserve = {clock_pin, select_pin}
        if mosi_pin is not None:
            to_reserve.add(mosi_pin)
        if miso_pin is not None:
            to_reserve.add(miso_pin)
        self.pin_factory.reserve_pins(self, *to_reserve)
        self._handle = lgpio.spi_open(port, device, self._baud, self._spi_flags)

    def _conflicts_with(self, other):
        return not (
            isinstance(other, LGPIOHardwareSPI) and
            (self._port, self._device) != (other._port, other._device)
            )

    def close(self):
        if not self.closed:
            lgpio.spi_close(self._handle)
        self._handle = None
        self.pin_factory.release_all(self)
        super(LGPIOHardwareSPI, self).close()

    @property
    def closed(self):
        return self._handle is None

    def __repr__(self):
        try:
            self._check_open()
            return 'SPI(port=%d, device=%d)' % (self._port, self._device)
        except DeviceClosed:
            return 'SPI(closed)'

    def _get_clock_mode(self):
        return self._spi_flags

    def _set_clock_mode(self, value):
        self._check_open()
        if not 0 <= value < 4:
            raise SPIInvalidClockMode("%d is not a valid SPI clock mode" % value)
        lgpio.spi_close(self._handle)
        self._spi_flags = value
        self._handle = lgpio.spi_open(
            self._port, self._device, self._baud, self._spi_flags)

    def _get_rate(self):
        return self._baud

    def _set_rate(self, value):
        self._check_open()
        value = int(value)
        lgpio.spi_close(self._handle)
        self._baud = value
        self._handle = lgpio.spi_open(
            self._port, self._device, self._baud, self._spi_flags)

    def read(self, n):
        self._check_open()
        count, data = lgpio.spi_read(self._handle, n)
        if count < 0:
            raise IOError('SPI transfer error %d' % count)
        return [int(b) for b in data]

    def write(self, data):
        self._check_open()
        count = lgpio.spi_write(self._handle, data)
        if count < 0:
            raise IOError('SPI transfer error %d' % count)
        return len(data)

    def transfer(self, data):
        self._check_open()
        count, data = lgpio.spi_xfer(self._handle, data)
        if count < 0:
            raise IOError('SPI transfer error %d' % count)
        return [int(b) for b in data]


class LGPIOHardwareSPIShared(SharedMixin, LGPIOHardwareSPI):
    @classmethod
    def _shared_key(cls, clock_pin, mosi_pin, miso_pin, select_pin, pin_factory):
        return (clock_pin, select_pin)



# vim: set fileencoding=utf-8:
#
# GPIO Zero: a library for controlling the Raspberry Pi's GPIO pins
#
# Copyright (c) 2015-2021 Dave Jones <dave@waveform.org.uk>
# Copyright (c) 2016 Andrew Scheller <github@loowis.durge.org>
#
# SPDX-License-Identifier: BSD-3-Clause

from __future__ import (
    unicode_literals,
    absolute_import,
    print_function,
    division,
    )
str = type('')

import warnings

from RPi import GPIO

from .local import LocalPiFactory, LocalPiPin
from ..exc import (
    PinInvalidFunction,
    PinSetInput,
    PinFixedPull,
    PinInvalidPull,
    PinInvalidState,
    PinInvalidBounce,
    PinPWMFixedValue,
    )


class RPiGPIOFactory(LocalPiFactory):
    """
    Extends :class:`~gpiozero.pins.local.LocalPiFactory`. Uses the `RPi.GPIO`_
    library to interface to the Pi's GPIO pins. This is the default pin
    implementation if the RPi.GPIO library is installed. Supports all features
    including PWM (via software).

    Because this is the default pin implementation you can use it simply by
    specifying an integer number for the pin in most operations, e.g.::

        from gpiozero import LED

        led = LED(12)

    However, you can also construct RPi.GPIO pins manually if you wish::

        from gpiozero.pins.rpigpio import RPiGPIOFactory
        from gpiozero import LED

        factory = RPiGPIOFactory()
        led = LED(12, pin_factory=factory)

    .. _RPi.GPIO: https://pypi.python.org/pypi/RPi.GPIO
    """

    def __init__(self):
        super(RPiGPIOFactory, self).__init__()
        GPIO.setmode(GPIO.BCM)
        GPIO.setwarnings(False)
        self.pin_class = RPiGPIOPin

    def close(self):
        super(RPiGPIOFactory, self).close()
        GPIO.cleanup()


class RPiGPIOPin(LocalPiPin):
    """
    Extends :class:`~gpiozero.pins.local.LocalPiPin`. Pin implementation for
    the `RPi.GPIO`_ library. See :class:`RPiGPIOFactory` for more information.

    .. _RPi.GPIO: https://pypi.python.org/pypi/RPi.GPIO
    """
    GPIO_FUNCTIONS = {
        'input':   GPIO.IN,
        'output':  GPIO.OUT,
        'i2c':     GPIO.I2C,
        'spi':     GPIO.SPI,
        'pwm':     GPIO.HARD_PWM,
        'serial':  GPIO.SERIAL,
        'unknown': GPIO.UNKNOWN,
        }

    GPIO_PULL_UPS = {
        'up':       GPIO.PUD_UP,
        'down':     GPIO.PUD_DOWN,
        'floating': GPIO.PUD_OFF,
        }

    GPIO_EDGES = {
        'both':    GPIO.BOTH,
        'rising':  GPIO.RISING,
        'falling': GPIO.FALLING,
        }

    GPIO_FUNCTION_NAMES = {v: k for (k, v) in GPIO_FUNCTIONS.items()}
    GPIO_PULL_UP_NAMES = {v: k for (k, v) in GPIO_PULL_UPS.items()}
    GPIO_EDGES_NAMES = {v: k for (k, v) in GPIO_EDGES.items()}

    def __init__(self, factory, number):
        super(RPiGPIOPin, self).__init__(factory, number)
        self._pull = 'up' if self.factory.pi_info.pulled_up(repr(self)) else 'floating'
        self._pwm = None
        self._frequency = None
        self._duty_cycle = None
        self._bounce = -666
        self._edges = GPIO.BOTH
        GPIO.setup(self.number, GPIO.IN, self.GPIO_PULL_UPS[self._pull])

    def close(self):
        self.frequency = None
        self.when_changed = None
        GPIO.cleanup(self.number)

    def output_with_state(self, state):
        self._pull = 'floating'
        GPIO.setup(self.number, GPIO.OUT, initial=state)

    def input_with_pull(self, pull):
        if pull != 'up' and self.factory.pi_info.pulled_up(repr(self)):
            raise PinFixedPull('%r has a physical pull-up resistor' % self)
        try:
            GPIO.setup(self.number, GPIO.IN, self.GPIO_PULL_UPS[pull])
            self._pull = pull
        except KeyError:
            raise PinInvalidPull('invalid pull "%s" for pin %r' % (pull, self))

    def _get_function(self):
        return self.GPIO_FUNCTION_NAMES[GPIO.gpio_function(self.number)]

    def _set_function(self, value):
        if value != 'input':
            self._pull = 'floating'
        if value in ('input', 'output') and value in self.GPIO_FUNCTIONS:
            GPIO.setup(self.number, self.GPIO_FUNCTIONS[value], self.GPIO_PULL_UPS[self._pull])
        else:
            raise PinInvalidFunction('invalid function "%s" for pin %r' % (value, self))

    def _get_state(self):
        if self._pwm:
            return self._duty_cycle
        else:
            return GPIO.input(self.number)

    def _set_state(self, value):
        if self._pwm:
            try:
                self._pwm.ChangeDutyCycle(value * 100)
            except ValueError:
                raise PinInvalidState('invalid state "%s" for pin %r' % (value, self))
            self._duty_cycle = value
        else:
            try:
                GPIO.output(self.number, value)
            except ValueError:
                raise PinInvalidState('invalid state "%s" for pin %r' % (value, self))
            except RuntimeError:
                raise PinSetInput('cannot set state of pin %r' % self)

    def _get_pull(self):
        return self._pull

    def _set_pull(self, value):
        if self.function != 'input':
            raise PinFixedPull('cannot set pull on non-input pin %r' % self)
        if value != 'up' and self.factory.pi_info.pulled_up(repr(self)):
            raise PinFixedPull('%r has a physical pull-up resistor' % self)
        try:
            GPIO.setup(self.number, GPIO.IN, self.GPIO_PULL_UPS[value])
            self._pull = value
        except KeyError:
            raise PinInvalidPull('invalid pull "%s" for pin %r' % (value, self))

    def _get_frequency(self):
        return self._frequency

    def _set_frequency(self, value):
        if self._frequency is None and value is not None:
            try:
                self._pwm = GPIO.PWM(self.number, value)
            except RuntimeError:
                raise PinPWMFixedValue('cannot start PWM on pin %r' % self)
            self._pwm.start(0)
            self._duty_cycle = 0
            self._frequency = value
        elif self._frequency is not None and value is not None:
            self._pwm.ChangeFrequency(value)
            self._frequency = value
        elif self._frequency is not None and value is None:
            self._pwm.stop()
            self._pwm = None
            self._duty_cycle = None
            self._frequency = None

    def _get_bounce(self):
        return None if self._bounce == -666 else (self._bounce / 1000)

    def _set_bounce(self, value):
        if value is not None and value < 0:
            raise PinInvalidBounce('bounce must be 0 or greater')
        f = self.when_changed
        self.when_changed = None
        try:
            self._bounce = -666 if value is None else int(value * 1000)
        finally:
            self.when_changed = f

    def _get_edges(self):
        return self.GPIO_EDGES_NAMES[self._edges]

    def _set_edges(self, value):
        f = self.when_changed
        self.when_changed = None
        try:
            self._edges = self.GPIO_EDGES[value]
        finally:
            self.when_changed = f

    def _call_when_changed(self, channel):
        super(RPiGPIOPin, self)._call_when_changed()

    def _enable_event_detect(self):
        GPIO.add_event_detect(
            self.number, self._edges,
            callback=self._call_when_changed,
            bouncetime=self._bounce)

    def _disable_event_detect(self):
        GPIO.remove_event_detect(self.number)
